/*!
 * \page ontocompiler_page The ontology compiler
 *
 * \section intro_sec Introduction
 * When building queries targeted at a given ontology, you'll likely find
 * yourself mirroring with Cubi classes the entire ontology. To solve this
 * problem, Cubi includes a small tool called cubi-oc, the Cubi ontology-compiler.
 * This tool will scan the available ontologies and generate a set of C++ headers
 * defining Cubi classes corresponding to the RDF resources of the ontology, which
 * you can include then in your source files.
 *
 * \note The Cubi ontology-compiler only works with the Tracker database. This
 * is due to the fact that Tracker offers some RDF extensions for ontology
 * introspection that are needed for the compiler to work properly.
 *
 * \note See http://www.tracker-project.org/ for more details on Tracker.
 *
 * \section generate_sec Generating the ontology headers
 * When run, the ontology compiler will create one folder called \c ontologies in the
 * current working directory, with inside one \c .h file for each introspected
 * ontology. Additionally, it will generate two files, \c ontologies.h which is
 * a global header to include all the ontologies, and \c ontologies.pri which
 * is a file that you can include in a .pro file if you use the qmake build system.
 *
 * \subsection generate_override_sec Overriding some of the ontology
 * You can override some properties defined in the ontology using a
 * \c cubi-oc.overrides file.
 *
 * The \c cubi-oc.overrides file has to be in the current working
 * directory when running the \c cubi-oc command.
 *
 * The syntax of the file follows the following format:
 * \code
 * [prefixedName]
 * Property = Value
 * \endcode
 *
 * The supported properties are:
 *
 * <table>
 *   <tr>
 *     <td>\c Domain</td><td>Override the domain of a RDF property</td>
 *   </tr>
 *   <tr>
 *     <td>\c Range</td><td>Override the range of a RDF property</td>
 *   </tr>
 * </table>
 *
 * An example section of an \c cubi-oc.overrides file could be :
 * \code
 * [nie:url]
 * Domain = http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#FileDataObject
 * \endcode
 *
 * \subsection generate_commandline_options Command line options reference
 * Run \c cubi-oc \c --help for more a short manual describing the various options.
 *
 * \section usage_sec Using the generated ontologies
 * The C++ classes describing the ontology information are generated in the
 * namespace \c Cubi::Ontologies . The C++ classes describing the RDF resources
 * in the ontology are generated in the \c Cubi::Resources namespace, in a
 * nested namespace with the same name as the ontology prefix. For example, the
 * C++ classes describing the RDF resources of the \c nco ontology will be in the
 * \c Cubi::Resources::nco namespace.
 *
 * \subsection usage_rdf_resources_sec Structure of the C++ class generated for any RDF resource
 * A C++ class describing an RDF resource (class, property or predefined
 * instance) has the following methods:
 *
 * <table>
 *   <tr>
 *     <td>\code static const char * encodedIri() \endcode</td>
 *     <td>
 *     Returns the full IRI of the RDF resource, in URL encoded form, for
 *     example \c http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Role
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code static const QString & prefixedName() \endcode</td>
 *     <td>
 *     Returns the prefixed name of the RDF resource, for example \c nco:Role
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code static const QString & iri() \endcode</td>
 *     <td>
 *     Helper function that returns a \c QString out of the result of encodedIri()
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code static const ResourceValue & resource() \endcode</td>
 *     <td>
 *     Helper function that returns a \c Cubi::ResourceValue representing the RDF
 *     class
 *     </td>
 *   </tr>
 * </table>
 *
 * Additionally, two types are defined inside the class:
 *
 * <table>
 *   <tr>
 *     <td>\code ResourceType \endcode</td>
 *     <td>
 *     Points to the RDF type of the resource, so for classes \c rdfs::Class
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code Ontology \endcode</td>
 *     <td>
 *     Points to the ontology of the resource, for example \c Ontologies::nco
 *     </td>
 *   </tr>
 * </table>
 *
 * \subsection usage_rdf_class_sec Structure of the C++ class generated for an RDF class
 * A C++ class describing an RDF class has the additional following types:
 *
 * <table>
 *   <tr>
 *     <td>\code Ontology \endcode</td>
 *     <td>
 *     Points to the ontology of the resource, for example \c Ontologies::nco
 *     </td>
 *   </tr>
 * </table>
 *
 * \subsection usage_rdf_properties_sec Structure of the C++ class generated for an RDF property
 * A C++ class describing an RDF property has the additional following methods:
 *
 * <table>
 *   <tr>
 *     <td>\code static const PredicateFunction & function() \endcode</td>
 *     <td>
 *     Returns a \c Cubi::PredicateFunction for that property
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code static bool singleValued() \endcode</td>
 *     <td>
 *     Returns \c true if the \c nrl:maxCardinality property of that RDF property
 *     is set to \c 1
 *     </td>
 *   </tr>
 * </table>
 *
 * The following types are also defined inside the class:
 *
 * <table>
 *   <tr>
 *     <td>\code ResourceType \endcode</td>
 *     <td>
 *     Points to the RDF type of the resource, so for properties
 *     \c rdfs::Property
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code Domain \endcode</td>
 *     <td>
 *     Points to the domain of the property
 *     </td>
 *   </tr>
 *   <tr>
 *     <td>\code Range \endcode</td>
 *     <td>
 *     Points to the range of the property
 *     </td>
 *   </tr>
 * </table>
 *
 * \subsection usage_rdf_instances_sec Structure of the C++ class generated for a predefined instance
 * Predefined instances have no additional methods or types defined inside the
 * C++ class compared to normal resources.
 */
