/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "blankvalue.h"

#include "utils.h"
#include "value_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class BlankValueData : public ValueData
{

public:
	BlankValueData();
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual Value::Type type() const;
	virtual bool isValid() const;

	QString m_name;
	bool m_auto;
};
/*! \endcond */

BlankValueData::BlankValueData()
	: ValueData()
	, m_auto(false)
{
}

ValueData*
BlankValueData::clone()
{
	return new BlankValueData(*this);
}

bool
BlankValueData::operator==(const ValueData *other) const
{
	const BlankValueData *otherData;

	if (other->type() != Value::TypeBlankValue) {
		return false;
	}

	otherData = static_cast<const BlankValueData*>(other);

	return m_name == otherData->m_name &&
	       m_auto == otherData->m_auto;
}

QString
BlankValueData::sparql() const
{
	static const QString sparqlTemplate = QString::fromLatin1("_:_%1");
	static const QString sparqlTemplateAuto = QString::fromLatin1("_:%1");

	const QString escapedName = Utils::escapeBlankName(m_name);

	return m_auto ? sparqlTemplateAuto.arg(escapedName) : sparqlTemplate.arg(escapedName);
}

Value::Type
BlankValueData::type() const
{
	return Value::TypeBlankValue;
}

bool
BlankValueData::isValid() const
{
	return not m_name.isNull();
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class BlankValue
 * Models a blank node.
 */

/*!
 * Constructs a blank node with an automatically assigned name.
 */
BlankValue::BlankValue()
	: Value(new BlankValueData)
{
	static int blankCounter = 0;
	static const QString nameTemplate = QString::fromLatin1("auto%1");
	data()->m_name = nameTemplate.arg(blankCounter++);
	data()->m_auto = true;
}

/*!
 * Constructs a blank node with a given name.
 */
BlankValue::BlankValue(const QString &name)
	: Value(new BlankValueData)
{
	data()->m_name = name;
}

/*!
 * Sets the node name.
 *
 * Note, the name in the generated Sparql will be prefixed with a _, to avoid
 * potential conflicts with automatically generated blank nodes.
 */
void
BlankValue::setName(const QString &name)
{
	data()->m_name = name;
	data()->m_auto = false;
}

/*!
 * Returns the node's name.
 */
QString
BlankValue::name() const
{
	return data()->m_name;
}

const BlankValueData*
BlankValue::data() const
{
	return static_cast<const BlankValueData *>(d.constData());
}

BlankValueData*
BlankValue::data()
{
	return static_cast<BlankValueData *>(d.data());
}
