/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "compositionalselect.h"

#include "patternbase_p.h"
#include "select.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class CompositionalSelectData : public PatternBaseData
{

public:
	PatternBaseData * clone();
	virtual bool operator==(const PatternBaseData *other) const;
	virtual bool isCompositionalSelect() const;
	virtual PatternBase::Type type() const;
	virtual bool isValid() const;
	virtual QString sparql(Options::SparqlOptions options) const;

	Select m_select;
};
/*! \endcond */

PatternBaseData*
CompositionalSelectData::clone()
{
	return new CompositionalSelectData(*this);
}

bool
CompositionalSelectData::operator==(const PatternBaseData *other) const
{
	const CompositionalSelectData *otherData = 0;

	if (type() != other->type()) {
		return false;
	}

	otherData = static_cast<const CompositionalSelectData*>(other);

	return m_select == otherData->m_select;
}

bool
CompositionalSelectData::isCompositionalSelect() const
{
	return true;
}

PatternBase::Type
CompositionalSelectData::type() const
{
	return PatternBase::TypeCompositionalSelect;
}

bool
CompositionalSelectData::isValid() const
{
	return true;
}

QString
CompositionalSelectData::sparql(Options::SparqlOptions options) const
{
	static const QString bracePattern = QString::fromLatin1("{%1}");
	return bracePattern.arg(m_select.sparql(options));
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class CompositionalSelect
 * Models a SELECT subquery used to define variables in a pattern.
 *
 * This class is mainly meant to be use as an adapter to add Select in a PatternBase.
 */

/*!
 * Constructs a CompositionalSelect from a Select
 */
CompositionalSelect::CompositionalSelect(const Select &select)
	: PatternBase(new CompositionalSelectData)
{
	data()->m_select = select;
}

/*!
 * Returns the Select of the CompositionalSelect
 */
Select
CompositionalSelect::select() const
{
	return data()->m_select;
}

const CompositionalSelectData*
CompositionalSelect::data() const
{
	return static_cast<const CompositionalSelectData *>(d.constData());
}

CompositionalSelectData*
CompositionalSelect::data()
{
	return static_cast<CompositionalSelectData *>(d.data());
}
