/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "filter.h"

#include "exists.h"
#include "literalvalue.h"
#include "function.h"
#include "select.h"
#include "value_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */

class FilterData : public ValueData
{

public:
	FilterData();
	FilterData(const FilterData &other);
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual Value::Type type() const;
	virtual bool isValid() const;

	Filter::Type m_type;
	QScopedPointer<Exists> m_exists;
	QScopedPointer<LiteralValue> m_literal;
	QScopedPointer<Function> m_function;
	QScopedPointer<Select> m_select;
};
/*! \endcond */

FilterData::FilterData()
	: ValueData()
{
}

FilterData::FilterData(const FilterData &other)
	: ValueData(other)
{
	// We need a copy constructor since QScopedPointer does not have one. Because
	// Filter only has const methods, the copy constructor should never be called,
	// so there's nothing to do here.
}

ValueData*
FilterData::clone()
{
	return new FilterData(*this);
}

bool
FilterData::operator==(const ValueData *other) const
{
	const FilterData *otherData = 0;

	if (other->type() != Value::TypeFilter) {
		return false;
	}

	otherData = static_cast<const FilterData*>(other);

	if (m_type != otherData->m_type) {
		return false;
	}

	switch(m_type) {
		case Filter::InvalidFilter:
			return true;
		case Filter::FunctionFilter:
			return *m_function == *otherData->m_function;
		case Filter::SelectFilter:
			return *m_select == *otherData->m_select;
		case Filter::ExistsFilter:
			return *m_exists == *otherData->m_exists;
		case Filter::LiteralFilter:
			return *m_literal == *otherData->m_literal;
		default:
			qFatal("%s: Unsupported type %d", Q_FUNC_INFO, m_type);
			return false;
	}
}

QString
FilterData::sparql() const
{
	static const QString braceTemplate = QString::fromLatin1("(%1)");

	switch(m_type) {
		case Filter::InvalidFilter:
			return QString();
		case Filter::FunctionFilter:
			return m_function->sparql();
		case Filter::SelectFilter:
			return braceTemplate.arg(m_select->sparql());
		case Filter::ExistsFilter:
			return m_exists->sparql();
		case Filter::LiteralFilter:
			return m_literal->sparql();
		default:
			qFatal("%s: Unsupported filter type %d", Q_FUNC_INFO, m_type);
			return QString();
	}
}

Value::Type
FilterData::type() const
{
	return Value::TypeFilter;
}

bool
FilterData::isValid() const
{
	switch(m_type) {
		case Filter::InvalidFilter:
			return false;
		case Filter::FunctionFilter:
			return m_function->isValid();
		case Filter::SelectFilter:
			return true;
		case Filter::ExistsFilter:
			return m_exists->isValid();
		case Filter::LiteralFilter:
			return m_literal->isValid();
		default:
			qFatal("%s: Unsupported filter type %d", Q_FUNC_INFO, m_type);
			return false;
	}
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Filter
 * Represents the contents of a FILTER in Sparql.
 *
 * A filter can either be a standard expression (like in <tt>FILTER(?var = 3)
 * </tt>), a subselect expression, an Exists block or a LiteralValue.
 * When using a Select as a filter, the Select must return only one column, for
 * example <tt>FILTER((SELECT COUNT(?t) WHERE {?res nao:hasTag ?t}) = 2)</tt>
 * to select only the solutions <tt>?t</tt> which have two tags.
 *
 * You can use filterType() to know if a Filter contains a Function, a Select,
 * an Exists or a LiteralValue.
 */

/*!
 * Constructs an invalid Filter.
 */
Filter::Filter()
	: Value(new FilterData)
{
	data()->m_type = InvalidFilter;
}

/*!
 * Constructs a Filter from an Exists.
 */
Filter::Filter(const Exists &exists)
	: Value(new FilterData)
{
	data()->m_exists.reset(new Exists(exists));
	data()->m_type = ExistsFilter;
}

/*!
 * Constructs a Filter from a Function.
 */
Filter::Filter(const Function &function)
	: Value(new FilterData)
{
	data()->m_function.reset(new Function(function));
	data()->m_type = FunctionFilter;
}

/*!
 * Constructs a Filter from a LiteralValue.
 */
Filter::Filter(const LiteralValue &literal)
	: Value(new FilterData)
{
	data()->m_literal.reset(new LiteralValue(literal));
	data()->m_type = LiteralFilter;
}

/*!
 * Constructs a Filter from a Select.
 */
Filter::Filter(const Select &select)
	: Value(new FilterData)
{
	data()->m_select.reset(new Select(select));
	data()->m_type = SelectFilter;
}

Filter::Type
Filter::filterType() const
{
	return data()->m_type;
}

Exists
Filter::exists() const
{
	if (data()->m_type != ExistsFilter) {
		qWarning("Trying to retrieve the EXISTS from a non-EXISTS filter");
	}

	if (not data()->m_exists.isNull()) {
		return *data()->m_exists;
	} else {
		return Exists();
	}
}

Function
Filter::function() const
{
	if (data()->m_type != FunctionFilter) {
		qWarning("Trying to retrieve the function from a non-function filter");
	}

	if (not data()->m_function.isNull()) {
		return *data()->m_function;
	} else {
		return Function();
	}
}

LiteralValue
Filter::literalValue() const
{
	if (data()->m_type != Filter::LiteralFilter) {
		qWarning("Trying to retrieve the literal from a non-literal filter");
	}

	if (not data()->m_literal.isNull()) {
		return *data()->m_literal;
	} else {
		return LiteralValue();
	}
}

Select
Filter::select() const
{
	if (data()->m_type != SelectFilter) {
		qWarning("Trying to retrieve the select from a non-select filter");
	}

	if (not data()->m_select.isNull()) {
		return *data()->m_select;
	} else {
		return Select();
	}
}

const FilterData*
Filter::data() const
{
	return static_cast<const FilterData *>(d.constData());
}

FilterData*
Filter::data()
{
	return static_cast<FilterData *>(d.data());
}

