/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "function.h"

#include "function_p.h"

CUBI_USE_NAMESPACE

ValueData*
FunctionData::clone()
{
	return new FunctionData(*this);
}

bool
FunctionData::operator==(const ValueData *other) const
{
	const FunctionData *otherData = 0;

	if (other->type() != Value::TypeFunction) {
		return false;
	}

	otherData = static_cast<const FunctionData*>(other);

	if (otherData->functionType() != Function::TypeInvalid) {
		return false;
	}

	return true;
}

Value::Type
FunctionData::type() const
{
	return Value::TypeFunction;
}

bool
FunctionData::isValid() const
{
	return false;
}

QString
FunctionData::sparql() const
{
	return QString();
}

Function::Type
FunctionData::functionType() const
{
	return Function::TypeInvalid;
}

///////////////////////////////////////////////////////////////////////////////

/*!
 * \class Function
 * Represents a generic function.
 *
 * The Function class is very versatile in Cubi. It can be used for predicate
 * functions in a projection, as an operator ("=" is an infix function with two
 * parameters), or as a normal function in a \c FILTER block.
 *
 * Generally speaking, Function represents a n-ary operator (but it's not as
 * scary as it looks).
 *
 * The Function class also holds the function parameters' values, that is why
 * apply() also returns a Function. If you have a PrefixFunction \c f, and call
 * <tt>f.apply(2)</tt>, you get <tt>f(2)</tt>, which is also a function. You
 * could then call PrefixFunction::apply() again with the value 3, and you
 * would get <tt>f(2, 3)</tt>.
 *
 * You will never use the Function class directly, but rather one of its
 * subclasses.
 */

/*!
 * Constructs an invalid Function.
 */
Function::Function()
	: Value(new FunctionData)
{
}

Function::Function(FunctionData *data)
	: Value(data)
{
}

/*!
 * Returns the type of the Function.
 */
Function::Type
Function::functionType() const
{
	return data()->functionType();
}

/*!
 * Replaces the function parameters with the contents of \p parameters.
 */
void
Function::setParameters(const ValueChain &parameters)
{
	data()->m_parameters = parameters;
}

/*!
 * Returns the currently set parameters.
 *
 * Note that the lib does not do any check on the number of parameters, you
 * just add as many parameters as you want.
 */
ValueChain
Function::parameters() const
{
	return data()->m_parameters;
}

const FunctionData*
Function::data() const
{
	return static_cast<const FunctionData *>(d.constData());
}

FunctionData*
Function::data()
{
	return static_cast<FunctionData *>(d.data());
}

