/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "graph.h"

#include "patterngroup_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class GraphData : public PatternGroupData
{

public:
	PatternBaseData * clone();
	virtual bool operator==(const PatternBaseData *other) const;
	virtual bool isGraph() const;
	virtual PatternBase::Type type() const;
	virtual bool isValid() const;
	virtual QString sparql(Options::SparqlOptions options) const;

	Graph::Type m_type;
	Value m_value;
};
/*! \endcond */

PatternBaseData*
GraphData::clone()
{
	return new GraphData(*this);
}

bool
GraphData::operator==(const PatternBaseData *other) const
{
	const GraphData *otherData = 0;

	if (type() != other->type()) {
		return false;
	}

	otherData = static_cast<const GraphData*>(other);

	if (m_type != otherData->m_type) {
		return false;
	}

	return PatternGroupData::operator==(other) &&
	       m_value == otherData->m_value;
}

bool
GraphData::isGraph() const
{
	return true;
}

PatternBase::Type
GraphData::type() const
{
	return PatternBase::TypeGraph;
}

bool
GraphData::isValid() const
{
	return m_value.isValid();
}

QString
GraphData::sparql(Options::SparqlOptions options) const
{
	static const QString graphTemplate = QString::fromLatin1("GRAPH %1 {%2}");
	static const QString graphTemplatePretty = QString::fromLatin1("GRAPH %1 {\n%2\n}");

	return (options & Options::PrettyPrint) ? graphTemplatePretty.arg(m_value.sparql(),
	                                                                  PatternGroupData::sparql(options))
	                                        : graphTemplate.arg(m_value.sparql(),
	                                                            PatternGroupData::sparql(options));
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Graph
 * Models a Sparql GRAPH contruct. There can be two type of GRAPH constructs,
 * one where you want to restrict the graph (in which case you'll set up a
 * fixed Iri for the Graph object, and type will be Type::IriGraph), and one
 * where you query the graph of a pattern, in this case you'll set a Variable
 * as a parameter of the Graph object, and type will be Type::VariableGraph.
 *
 * Example of a graph with a fixed IRI:
 * <code>GRAPH <fixed:iri> {?c a nco:PersonContact}</code>
 *
 * Example of a graph with a variable:
 * <code>GRAPH ?graph {?c a nco:PersonContact}</code>
 */

/*!
 * Constructs a graph with an IRI.
 */
Graph::Graph(const ResourceValue &iri)
	: PatternGroup(new GraphData)
{
	data()->m_value = iri;
	data()->m_type = IriGraph;
}

/*!
 * Constructs a graph with a variable.
 */
Graph::Graph(const Variable &variable)
	: PatternGroup(new GraphData)
{
	data()->m_value = variable;
	data()->m_type = VariableGraph;
}

/*!
 * Returns the type of the graph.
 */
Graph::Type
Graph::graphType() const
{
	return data()->m_type;
}

/*!
 * Sets the IRI of the graph.
 */
void
Graph::setIri(const ResourceValue &iri)
{
	data()->m_value = iri;
	data()->m_type = IriGraph;
}

/*!
 * Sets the variable of the graph.
 */
void
Graph::setVariable(const Variable &variable)
{
	data()->m_value = variable;
	data()->m_type = VariableGraph;
}

/*!
 * Returns the IRI of the graph.
 *
 * If the type of the graph is not IriGraph, returns an invalid ResourceValue.
 */
ResourceValue
Graph::iri() const
{
	if (data()->m_type != IriGraph) {
		qWarning("Trying to retrieve the IRI from a Variable Graph");
		return ResourceValue();
	}

	const ResourceValue &value = static_cast<const ResourceValue&>(data()->m_value);

	return value;
}

/*!
 * Returns the variable of the graph.
 *
 * If the type of the graph is not VariableGraph, returns an invalid Variable.
 */
Variable
Graph::variable() const
{
	if (data()->m_type != VariableGraph) {
		qWarning("Trying to retrieve the Variable from a Iri Graph");
		return Variable::invalid();
	}

	const Variable &variable = static_cast<const Variable&>(data()->m_value);

	return variable;
}

const GraphData*
Graph::data() const
{
	return static_cast<const GraphData *>(d.constData());
}

GraphData*
Graph::data()
{
	return static_cast<GraphData *>(d.data());
}
