/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "infixfunction.h"

#include "function_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class InfixFunctionData : public FunctionData
{

public:
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual bool isValid() const;

	virtual Function::Type functionType() const;

	QString m_name;
};
/*! \endcond */

ValueData*
InfixFunctionData::clone()
{
	return new InfixFunctionData(*this);
}

bool
InfixFunctionData::operator==(const ValueData *other) const
{
	const InfixFunctionData *otherData = 0;

	if (other->type() != Value::TypeFunction) {
		return false;
	}

	if (static_cast<const FunctionData*>(other)->functionType() !=
	    Function::TypeInfixFunction) {
		return false;
	}

	otherData = static_cast<const InfixFunctionData*>(other);

	return m_name == otherData->m_name &&
	       m_parameters == otherData->m_parameters;
}

QString
InfixFunctionData::sparql() const
{
	static const QString nullString;
	static const QString operatorTemplate = QString::fromLatin1(" %1 ");
	static const QString parenthesisTemplate = QString::fromLatin1("(%1)");
	QStringList params;

	if (m_parameters.empty()) {
		return nullString;
	}

	// This is to avoid excessive generation of parenthesis
	if (m_parameters.size() == 1) {
		return m_parameters.first().sparql();
	}

	foreach(const Value &v, m_parameters) {
		const QString &sparql = v.sparql();

		if (sparql.isEmpty()) {
			continue;
		}

		params.append(sparql);
	}

	return parenthesisTemplate.arg(params.join(operatorTemplate.arg(m_name)));
}

bool
InfixFunctionData::isValid() const
{
	return not m_name.isNull();
}

Function::Type
InfixFunctionData::functionType() const
{
	return Function::TypeInfixFunction;
}

CUBI_END_NAMESPACE

CUBI_USE_NAMESPACE

/*!
 * \class InfixFunction
 * Represents an infix function.
 *
 * Infix functions are generally used for Sparql operators like <tt>=, >, IN</tt>...
 */

/*!
 * Constructs an infix function from its \p name.
 */
InfixFunction::InfixFunction(const QString &name)
	: Function(new InfixFunctionData)
{
	data()->m_name = name;
}

bool
InfixFunction::operator==(const InfixFunction &other) const
{
	return data()->m_name == other.data()->m_name;
}

/*!
 * Sets the name of the function.
 */
void
InfixFunction::setName(const QString &name)
{
	data()->m_name = name;
}

/*!
 * Returns the name of the function.
 */
QString
InfixFunction::name() const
{
	return data()->m_name;
}

/*!
 * Applies the given \p values as parameters of the function.
 *
 * Returns a new InfixFunction with the parameters set (see Function
 * documentation).
 */
InfixFunction
InfixFunction::apply(const ValueChain &values) const
{
	InfixFunction f = *this;

	f.data()->m_parameters = values;

	return f;
}

/*!
 * Applies the function to \p v1.
 *
 * Returns a new InfixFunction with the parameters set (see Function
 * documentation).
 */
InfixFunction
InfixFunction::apply(const Value &v1) const
{
	ValueChain params;
	params << v1;
	return apply(params);
}

/*!
 * Applies the function to \p v1 and \p v2.
 *
 * Returns a new InfixFunction with the parameters set (see Function
 * documentation).
 */
InfixFunction
InfixFunction::apply(const Value &v1, const Value &v2) const
{
	ValueChain params;
	params << v1;
	params << v2;
	return apply(params);
}

/*!
 * Applies the function to \p v1, \p v2 and \p v3.
 *
 * Returns a new InfixFunction with the parameters set (see Function
 * documentation).
 */
InfixFunction
InfixFunction::apply(const Value &v1, const Value &v2, const Value &v3) const
{
	ValueChain params;
	params << v1;
	params << v2;
	params << v3;
	return apply(params);
}

const InfixFunctionData*
InfixFunction::data() const
{
	return static_cast<const InfixFunctionData *>(d.constData());
}

InfixFunctionData*
InfixFunction::data()
{
	return static_cast<InfixFunctionData *>(d.data());
}
