/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "insert.h"

#include "updatebase_p.h"
#include "utils.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class InsertData : public UpdateBaseData
{

public:
	virtual UpdateBaseData * clone();
	virtual UpdateBase::Type type() const;
	virtual QString sparql(Options::SparqlOptions options = Options::DefaultSparqlOptions) const;

	Insert::ConflictStrategy strategy;
};
/*! \endcond */

UpdateBaseData*
InsertData::clone()
{
	return new InsertData(*this);
}

UpdateBase::Type
InsertData::type() const
{
	return UpdateBase::InsertUpdate;
}

QString
InsertData::sparql(Options::SparqlOptions options) const
{
	static const QString insertTemplateCompact = QString::fromLatin1("INSERT%1%2 {%3}%4");
	static const QString insertTemplatePretty = QString::fromLatin1("INSERT%1%2 {\n%3\n}%4");
	static const QString replaceTemplate = QString::fromLatin1(" OR REPLACE");
	static const QString intoTemplate = QString::fromLatin1(" INTO %1");
	static const QString whereTemplateCompact = QString::fromLatin1(" WHERE {%1}");
	static const QString whereTemplatePretty = QString::fromLatin1(" WHERE {\n%1\n}");

	const QString &insertTemplate = (options & Options::PrettyPrint) ? insertTemplatePretty
	                                                                 : insertTemplateCompact;
	const QString &whereTemplate = (options & Options::PrettyPrint) ? whereTemplatePretty
	                                                                : whereTemplateCompact;

	const QString q = insertTemplate.arg(Insert::Replace == strategy ? replaceTemplate : QString(),
	                                     graph.isValid() ? intoTemplate.arg(graph.sparql()) : QString(),
	                                     data.sparql(options),
	                                     restrictions.patterns().empty() ? QString()
	                                                                        : whereTemplate.arg(restrictions.sparql(options)));
	return (options & Options::PrettyPrint) ? Utils::prettyPrint(q) : q;
}

CUBI_END_NAMESPACE

CUBI_USE_NAMESPACE

/*!
 * \class Insert
 * Models an INSERT statement in SPARQL.
 *
 * A Insert statement has two parts, the first one, called the "data" part,
 * being the graph pattern that has to be inserted, and the second one,
 * called the "restriction" part, being the pattern that must be matched
 * for the actual insertion to happen. The first part can be manipulated
 * using the UpdateBase::addData() functions, while the second part is
 * manipulated using the UpdateBase::addRestriction() functions.
 *
 * Most of the useful functions of this class are defined in UpdateBase, which
 * is the common base to Delete and Insert (since they share a very similar
 * structure).
 */

/*!
 * Constructs an empty INSERT statement.
 *
 * An optional \p graph can be set, if you want to generate a \c INSERT \c INTO
 * \c ... query.
 *
 * \deprecated \c INSERT \c INTO variant is deprecated. New code should create a
 * Graph and add it using UpdateBase::addData().
 */
Insert::Insert(const ResourceValue &graph)
	: UpdateBase(new InsertData)
{
	d->graph = graph;
	data()->strategy = Abort;
}

/*!
 * Constructs an empty INSERT statement.
 *
 * An optional \p graph can be set, if you want to generate a \c INSERT \c INTO
 * \c ... query.
 *
 * The \p strategy argument permits switching between regular \c INSERT that
 * aborts if existing resources have conflicting data in single-value properties,
 * and \c INSERT \c OR \c REPLACE which overwrites values for single valued
 * properties.
 *
 * \deprecated \c INSERT \c INTO variant is deprecated. New code should create a
 * Graph and add it using UpdateBase::addData().
 */
Insert::Insert(ConflictStrategy strategy, const ResourceValue &graph)
	: UpdateBase(new InsertData)
{
	d->graph = graph;
	data()->strategy = strategy;
}

/*!
 * Constructs an empty INSERT statement.
 *
 * The \p strategy argument permits switching between regular \c INSERT that
 * aborts if existing resources have conflicting data in single-value properties,
 * and \c INSERT \c OR \c REPLACE which overwrites values for single valued
 * properties.
 *
 * \deprecated \c INSERT \c INTO variant is deprecated. New code should create a
 * Graph and add it using UpdateBase::addData().
 */
Insert::Insert(ConflictStrategy strategy)
	: UpdateBase(new InsertData)
{
	data()->strategy = strategy;
}

void
Insert::setConflictStrategy(ConflictStrategy strategy)
{
	data()->strategy = strategy;
}

Insert::ConflictStrategy
Insert::conflictStrategy() const
{
	return data()->strategy;
}

const InsertData*
Insert::data() const
{
	return static_cast<const InsertData *>(d.constData());
}

InsertData*
Insert::data()
{
	return static_cast<InsertData *>(d.data());
}
