/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "literalvalue.h"

#include "value_p.h"
#include "valuelist.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class LiteralValueData : public ValueData
{

public:
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual Value::Type type() const;
	virtual bool isValid() const;

	QVariant m_value;
};
/*! \endcond */

ValueData*
LiteralValueData::clone()
{
	return new LiteralValueData(*this);
}

bool
LiteralValueData::operator==(const ValueData *other) const
{
	if (other->type() != Value::TypeLiteralValue) {
		return false;
	}

	return m_value == static_cast<const LiteralValueData*>(other)->m_value;
}


static QString
sparqlEscape(const QString &original)
{
	QString escaped;
	escaped.reserve(original.size());

	for(int i = 0; i < original.size(); ++i) {
		switch(original.at(i).toLatin1()) {
			case '\t':
                escaped.append(QString::fromLatin1("\\t"));
				break;
			case '\n':
                escaped.append(QString::fromLatin1("\\n"));
				break;
			case '\r':
                escaped.append(QString::fromLatin1("\\r"));
				break;
			case '"':
                escaped.append(QString::fromLatin1("\\\""));
				break;
			case '\\':
                escaped.append(QString::fromLatin1("\\\\"));
				break;
			default:
				escaped.append(original.at(i));
		}
	}

	return escaped;
}

QString
LiteralValueData::sparql() const
{
	static const QString stringTemplate = QString::fromLatin1("\"%1\"");

	switch(m_value.type()) {
		case QVariant::Bool:
		{
			static const QString trueString = QString::fromLatin1("true");
			static const QString falseString = QString::fromLatin1("false");
			return m_value.toBool() ? trueString : falseString;
		}
		case QVariant::Date:
		case QVariant::DateTime:
		{
			const QDateTime dateTimeValue = m_value.toDateTime();
			const QString dateTimeString = dateTimeValue.toString(Qt::ISODate);
			return stringTemplate.arg(dateTimeString);
		}
		case QVariant::Double:
		case QVariant::Int:
		case QVariant::LongLong:
		case QVariant::UInt:
		case QVariant::ULongLong:
			return m_value.toString();
		case QVariant::Char:
		case QVariant::String:
		case QVariant::Url:
			return stringTemplate.arg(sparqlEscape(m_value.toString()));
		case QVariant::StringList:
		case QVariant::List:
		{
			ValueList list;
			foreach(const QVariant &var, m_value.toList()) {
				list.addValue(LiteralValue(var));
			}

			return list.sparql();
		}
		default:
			qWarning("LiteralValue::sparql: Unsupported data type %d", m_value.type());
			return QString();
	}
}

Value::Type
LiteralValueData::type() const
{
	return Value::TypeLiteralValue;
}

bool
LiteralValueData::isValid() const
{
	if (not m_value.isValid()) {
		return false;
	}

	switch(m_value.type()) {
		case QVariant::Bool:
		case QVariant::Char:
		case QVariant::Date:
		case QVariant::DateTime:
		case QVariant::Double:
		case QVariant::Int:
		case QVariant::LongLong:
		case QVariant::UInt:
		case QVariant::ULongLong:
		case QVariant::String:
		case QVariant::List:
		case QVariant::StringList:
		case QVariant::Url:
			return true;
		default:
			return false;
	}
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class LiteralValue
 * Models a literal value in Sparql
 *
 * The LiteralValue class could be seen as a QVariant with a sparql() function.
 * It can hold any type supported by QVariant, but only certains of those types
 * can be serialized into proper Sparql.
 *
 * The list of those types is:
 * \li QVariant::Bool
 * \li QVariant::Char
 * \li QVariant::Date
 * \li QVariant::DateTime
 * \li QVariant::Double
 * \li QVariant::Int
 * \li QVariant::LongLong
 * \li QVariant::UInt
 * \li QVariant::ULongLong
 * \li QVariant::String
 * \li QVariant::Url
 *
 * LiteralValue also handles lists, however be warned that those are treated as
 * sets in Sparql, that is, duplicates will be removed, and conservation of
 * order is not guaranteed. A list in a LiteralValue models a multiple value
 * in Sparql.
 *
 */

/*!
 * Constructs an invalid LiteralValue.
 */
LiteralValue::LiteralValue()
	: Value(new LiteralValueData)
{
}

/*!
 * Constructs a LiteralValue from a QVariant.
 */
LiteralValue::LiteralValue(const QVariant &value)
	: Value(new LiteralValueData)
{
	data()->m_value = value;
}

/*!
 * Sets the value of the LiteralValue.
 */
void
LiteralValue::setValue(const QVariant &value)
{
	data()->m_value = value;
}

/*!
 * Returns the value of the LiteralVariant.
 */
QVariant
LiteralValue::value() const
{
	return data()->m_value;
}

const LiteralValueData*
LiteralValue::data() const
{
	return static_cast<const LiteralValueData *>(d.constData());
}

LiteralValueData*
LiteralValue::data()
{
	return static_cast<LiteralValueData *>(d.data());
}

