/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "pattern.h"

#include "patternbase_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class PatternData : public PatternBaseData
{

public:
	virtual PatternBaseData * clone();
	virtual bool operator==(const PatternBaseData *other) const;
	virtual bool isPattern() const;
	virtual PatternBase::Type type() const;
	virtual bool isValid() const;
	virtual QString sparql(Options::SparqlOptions options) const;

	Value m_subject;
	Value m_predicate;
	Value m_object;
};
/*! \endcond */

PatternBaseData*
PatternData::clone()
{
	return new PatternData(*this);
}

bool
PatternData::operator==(const PatternBaseData *other) const
{
	const PatternData *otherData = 0;

	if (type() != other->type()) {
		return false;
	}

	otherData = static_cast<const PatternData*>(other);

	return m_subject   == otherData->m_subject &&
	       m_predicate == otherData->m_predicate &&
	       m_object    == otherData->m_object;
}

bool
PatternData::isPattern() const
{
	return true;
}

PatternBase::Type
PatternData::type() const
{
	return PatternBase::TypePattern;
}

bool
PatternData::isValid() const
{
	return m_subject.isValid() && m_predicate.isValid() && m_object.isValid();
}

QString
PatternData::sparql(Options::SparqlOptions /* options */) const
{
	static const QString templateString = QString::fromLatin1("%1 %2 %3");
	return templateString.arg(m_subject.sparql(),
	                          m_predicate.sparql(),
	                          m_object.sparql());
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Pattern
 * Models an RDF triple.
 *
 * The pattern class models an RDF triple, that is a subject, a predicate and
 * an object.
 *
 * You will likely not use this class directly but rather functions like
 * Select::addRestriction(const Value *s, const Value *p, const Value *o),
 * PatternGroup::addPattern(const Value *s, const Value *p, const Value *o),
 * etc.
 *
 */

/*!
 * Constructs an invalid pattern.
 */
Pattern::Pattern()
	: PatternBase(new PatternData)
{
}

/*!
 * Constructs a Pattern.
 */
Pattern::Pattern(const Value &s, const Value &p, const Value &o)
	: PatternBase(new PatternData)
{
	data()->m_subject = s;
	data()->m_predicate = p;
	data()->m_object = o;
}

bool
Pattern::operator==(const Pattern &other) const
{
	if ((subject() == other.subject()) &&
	    (predicate() == other.predicate()) &&
	    (object() == other.object())) {
		return true;
	}

	return false;
}

bool
Pattern::operator!=(const Pattern &other) const
{
	return not (*this == other);
}

/*!
 * Sets the subject of the triple.
 */
void
Pattern::setSubject(const Value &subject)
{
	data()->m_subject = subject;
}

/*!
 * Returns the subject of the triple.
 */
Value
Pattern::subject() const
{
	return data()->m_subject;
}

/*!
 * Sets the predicate of the triple.
 */
void
Pattern::setPredicate(const Value &predicate)
{
	data()->m_predicate = predicate;
}

/*!
 * Returns the predicate of the triple.
 */
Value
Pattern::predicate() const
{
	return data()->m_predicate;
}

/*!
 * Sets the object of the triple.
 */
void
Pattern::setObject(const Value &object)
{
	data()->m_object = object;
}

/*!
 * Returns the object of the triple.
 */
Value
Pattern::object() const
{
	return data()->m_object;
}

const PatternData*
Pattern::data() const
{
	return static_cast<const PatternData *>(d.constData());
}

PatternData*
Pattern::data()
{
	return static_cast<PatternData *>(d.data());
}
