/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "patterngroup.h"

#include "filter.h"
#include "generatedvariable.h"
#include "pattern.h"
#include "patterngroup_p.h"

CUBI_USE_NAMESPACE

int PatternGroupData::m_tmpVarIndex = 0;

PatternGroupData::PatternGroupData()
	: PatternBaseData()
	, m_optional(false)
{
}

PatternBaseData*
PatternGroupData::clone()
{
	return new PatternGroupData(*this);
}

bool
PatternGroupData::operator==(const PatternBaseData *other) const
{
	const PatternGroupData *otherData = 0;

	if (type() != other->type()) {
		return false;
	}

	otherData = static_cast<const PatternGroupData*>(other);

	return m_optional == otherData->m_optional &&
	       m_patterns == otherData->m_patterns &&
	       m_filter == otherData->m_filter;
}

bool
PatternGroupData::isPatternGroup() const
{
	return true;
}

PatternBase::Type
PatternGroupData::type() const
{
	return PatternBase::TypePatternGroup;
}

bool
PatternGroupData::isValid() const
{
	return true;
}

/*
 * This function takes a hierarchy of PatternBase, and flattens it as much as
 * possible. So if we have A containing 3 patterns w x y and B containg A and
 * a pattern z (would be B[ A[ w x y ] z]), flattening would take all the
 * patterns of A, put them in B and remove A (final result would be
 * B[ w x y z ]).
 *
 * Because PatternGroup uses copy and write, you cannot have cycles, ie A
 * including B and B including A. Therefore, we have a true hierarchy and are
 * assured that the function will always return.
 */
static void
flatten(const QList<PatternBase> &patterns, QList<PatternBase> &dest)
{
	foreach(const PatternBase &base, patterns) {
		if (base.type() == PatternBase::TypePatternGroup) {
			const PatternGroup &g = static_cast<const PatternGroup &>(base);

			if (not (g.type() == PatternBase::TypeGraph)) {
				if (not g.optional()) {
					flatten(g.patterns(), dest);
				} else {
					PatternGroup optional = g;
					QList<PatternBase> patterns;

					flatten(g.patterns(), patterns);
					optional.setOptional(true);
					optional.setPatterns(patterns);
					optional.setFilter(Filter(Function()));
					dest.append(optional);
				}

				if (g.filter().isValid()) {
					PatternGroup f;
					f.setFilter(g.filter());
					dest.append(f);
				}
			} else {
				dest.append(base);
			}
		} else {
			dest.append(base);
		}
	}
}

// Walks the list of patterns and removes the duplicate ones
static void
removeDuplicatePatterns(QList<PatternBase> &patterns)
{
	QSet<QString> knownPatterns;

	// Remove duplicate patterns
	for(int i = 0; i < patterns.size(); i++) {
		const PatternBase &base = patterns.at(i);
		QString sparql = base.sparql(Options::NoOptimization);

		if (knownPatterns.contains(sparql)) {
			patterns.removeAt(i--);
		} else {
			knownPatterns.insert(sparql);
		}
	}
}

static QList<QList<Pattern> >
listChains(const PatternGroup &group)
{
	QList<QList<Pattern> > chains;
	QList<Pattern> lastChain;

	Pattern lastPattern;

	foreach(const PatternBase &base, group.patterns()) {
		if (base.type() != PatternBase::TypePattern) {
			continue;
		}

		const Pattern &p = static_cast<const Pattern&>(base);

		if (not lastPattern.isValid() || lastPattern.object() != p.subject()) {
			chains.append(QList<Pattern>() << p);
		}

		lastPattern = p;

	}

	return chains;
}

static bool
removePattern(PatternGroup &group, const Pattern &pattern)
{
	QList<PatternBase> patterns = group.patterns();

	for(int i = 0; i < patterns.size(); ++i) {
		if (patterns.at(i) == pattern) {
			patterns.removeAt(i--);

			group.setPatterns(patterns);
			return true;
		}
	}

	return false;
}

static void
nestOptionals(QList<PatternBase> &patterns)
{
	QMultiHash<Pattern, int> prefixes;
	QList<PatternGroup> newGroups;

	// First walk the patterns to register the potential factorizations
	for(int i = 0; i < patterns.size(); i++) {
		const PatternBase &base = patterns.at(i);

		if (base.type() != PatternBase::TypePatternGroup) {
			continue;
		}

		const PatternGroup &group = static_cast<const PatternGroup&>(base);

		if (not group.optional()) {
			continue;
		}

		const QList<QList<Pattern> > chains = listChains(group);

		foreach(const QList<Pattern> &chain, chains) {
			if (chain.size() == 1) {
				prefixes.insert(chain.first() , i);
			}
		}
	}

	// Prune all patterns that can't be factorized
	foreach(const Pattern &pattern, prefixes.keys()) {
		if (not (prefixes.values(pattern).size() > 1)) {
			prefixes.remove(pattern);
		}
	}

	// Create the nested OPTIONALs
	foreach(const Pattern &pattern, prefixes.uniqueKeys()) {
		PatternGroup group;
		group.setOptional(true);
		group.addPattern(pattern);

		QList<int> patternIndices = prefixes.values(pattern);
		qSort(patternIndices);

		foreach(int i, patternIndices) {
			PatternGroup g = static_cast<const PatternGroup&>(patterns.at(i));
			// removePattern only removes the first matching pattern, but since we
			// require duplicate removal for OPTIONAL nesting, we can safely assume
			// there is only one
			removePattern(g, pattern);

			// If our pattern group contains only one OPTIONAL pattern, and we
			// nest it into another OPTIONAL pattern, then we will generate
			// something like "OPTIONAL { OPTIONAL { a b c } }, which we don't
			// really want
			if (g.patterns().size() == 1 &&
			    g.patterns().first().type() == PatternBase::TypePatternGroup &&
			    static_cast<const PatternGroup&>(g.patterns().first()).optional()) {
				g.setOptional(false);
			}

			group.addPattern(g);
		}

		newGroups.append(group);
	}

	// Remove the old unnested patterns
	QList<int> indicesToRemove = prefixes.values();
	qSort(indicesToRemove.begin(), indicesToRemove.end(), qGreater<int>());

	foreach(int i, indicesToRemove) {
		patterns.removeAt(i);
	}

	// And add the new nested ones
	for(int i = 0; i < newGroups.size(); ++i) {
		const PatternGroup &group = newGroups.at(i);
		patterns.append(group);
	}
}

// Regroup patterns by their predicate, so we can share predicates as much
// as possible
// ** This method requires that all patterns have the same subject **
static void
regroupPredicates(QList<Pattern> &patterns)
{
	if (patterns.empty()) {
		return;
	}

	QList<Pattern> groupedPatterns;
	QHash<QString, QList<Value> > predicateValues;

	foreach (const Pattern &pattern, patterns) {
		const QString predicateSparql = pattern.predicate().sparql();
		predicateValues[predicateSparql].append(pattern.object());
	}

	foreach (Pattern pattern, patterns) {
		const QString predicateSparql = pattern.predicate().sparql();

		if (not predicateValues.contains(predicateSparql)) {
			continue;
		}

		foreach (const Value &value, predicateValues[predicateSparql]) {
			// We copy the original pattern instead of creating a new one to preserve
			// any other info that might have been set
			pattern.setObject(value);
			groupedPatterns.append(pattern);
		}

		// We added all the values for that predicate, so we now remove it
		// from the hash so it won't be picked by later iterations
		predicateValues.remove(predicateSparql);
	}

	patterns = groupedPatterns;
}

// Regroups patterns by their subject. It preserves the initial order of
// subjects in the query.
static void
regroupPatterns(QList<PatternBase> &patterns)
{
	QList<PatternBase> groupedPatterns;
	QHash<QString, QList<Pattern> > patternsBySubject;

	// First walk all the patterns to identify the subjects and their patterns
	foreach (const PatternBase &base, patterns) {
		if (base.type() != PatternBase::TypePattern) {
			continue;
		}

		const Pattern &pattern = static_cast<const Pattern&>(base);
		const QString subjectSparql = pattern.subject().sparql();

		patternsBySubject[subjectSparql].append(pattern);
	}

	// The output the list grouped
	foreach (const PatternBase &base, patterns) {
		if (base.type() != PatternBase::TypePattern) {
			groupedPatterns.append(base);
			continue;
		}

		const Pattern &pattern = static_cast<const Pattern&>(base);
		const QString subjectSparql = pattern.subject().sparql();

		if (not patternsBySubject.contains(subjectSparql)) {
			continue;
		}

		QList<Pattern> patterns = patternsBySubject[subjectSparql];
		regroupPredicates(patterns);

		foreach (const PatternBase &base, patterns) {
			groupedPatterns.append(base);
		}

		// We added all the predicates for that subject, so we now remove it
		// from the hash so it won't be picked by later iterations
		patternsBySubject.remove(subjectSparql);
	}

	patterns = groupedPatterns;
}

QString
PatternGroupData::sparql(Options::SparqlOptions options) const
{
	static const QString optionalTemplate = QString::fromLatin1("OPTIONAL {%1}");
	static const QString optionalTemplatePretty = QString::fromLatin1("OPTIONAL {\n%1\n}");
	static const QString filterTemplate = QString::fromLatin1("FILTER(%1)");
	// Those templates don't really belong here :-/
	static const QString predicateJoin = QString::fromLatin1(" ; ");
	static const QString valueJoin = QString::fromLatin1(", ");
	static const QString predicateObjectTemplate = QString::fromLatin1("%1 %2");

	QString query;
	QStringList patternsSparql;
	QList<PatternBase> flattenedPatterns;

	if (options & (Options::RemoveDuplicates | Options::NestOptionals)) {
		flatten(m_patterns, flattenedPatterns);
	} else {
		flattenedPatterns = m_patterns;
	}

	// OPTIONAL nesting needs duplicate removal to work properly
	if (options & (Options::RemoveDuplicates | Options::NestOptionals)) {
		removeDuplicatePatterns(flattenedPatterns);
	}

	if (options & Options::NestOptionals) {
		nestOptionals(flattenedPatterns);
	}

	if (options & Options::GroupPatterns) {
		regroupPatterns(flattenedPatterns);
	}

	// We use two different type of separators ("joins") to avoid unnecessary "."
	// between normal patterns and some pattern groups like GRAPH or OPTIONAL
	const QString &join = QLatin1String((options & Options::PrettyPrint) ? " .\n" : " . ");
	const QString &groupJoin = QLatin1String((options & Options::PrettyPrint) ? "\n" : " ");
	const int lastIndex = flattenedPatterns.size() - 1;
	patternsSparql.reserve(2 * flattenedPatterns.size() - 1);
	Value lastSubject;
	Value lastPredicate;

	for (int i = 0; i < flattenedPatterns.size(); ++i) {
		const PatternBase &p = flattenedPatterns.at(i);
		QString sparql;

		if (p.type() == PatternBase::TypePattern && (options & Options::GroupPatterns)) {
			const Pattern &pattern = static_cast<const Pattern&>(p);
			const Value subject = pattern.subject();

			if (subject.isValid() && subject == lastSubject) {
				patternsSparql.removeLast(); // Remove last join

				if (pattern.predicate() == lastPredicate) {
					patternsSparql.append(valueJoin);
					sparql = pattern.object().sparql();
				} else {
					patternsSparql.append(predicateJoin);
					sparql = predicateObjectTemplate.arg(pattern.predicate().sparql(),
					                                     pattern.object().sparql());
				}
			} else {
				lastSubject = subject;
				sparql = pattern.sparql(options);
			}

			lastPredicate = pattern.predicate();
		} else {
			sparql = p.sparql(options);
		}

		if (sparql.isEmpty()) {
			continue;
		}

		patternsSparql << sparql;

		if (i == lastIndex) {
			break;
		}

		const PatternBase::Type type = flattenedPatterns.at(i+1).type();
		if (i < lastIndex && not (type == PatternBase::TypePatternGroup ||
		                          type == PatternBase::TypeGraph ||
		                          type == PatternBase::TypeExists)) {
			patternsSparql << join;
		} else {
			patternsSparql << groupJoin;
		}
	}

	query = patternsSparql.join(QString());

	if (m_filter.isValid()) {
		if (!query.isEmpty()) {
			query.append(groupJoin);
		}
		query.append(filterTemplate.arg(m_filter.sparql()));
	}

	// Avoid returning "OPTIONAL { }"
	if (query.isEmpty()) {
		return QString();
	}

	return m_optional ? ((options & Options::PrettyPrint) ? optionalTemplatePretty.arg(query)
	                                                      : optionalTemplate.arg(query))
	                  : query;
}

///////////////////////////////////////////////////////////////////////////////

/*!
 * \class PatternGroup
 *
 * Models a graph pattern (a set of triples).
 *
 * The PatternGroup class can be used to group triples together. Grouping
 * triples together can be useful if you want to define some of them as
 * optional, or apply filtering only to a set of triples.
 */

/*!
 * Constructs an empty PatternGroup.
 */
PatternGroup::PatternGroup()
	: PatternBase(new PatternGroupData)
{
}

PatternGroup::PatternGroup(PatternGroupData *data)
	: PatternBase(data)
{
}

PatternGroup::~PatternGroup()
{
}

/*!
 * Returns \c true if the group is optional, \c false else.
 */
bool
PatternGroup::optional() const
{
	return data()->m_optional;
}

/*!
 * Sets the optionality of this group.
 *
 * The generated Sparql will be enclosed in an \c OPTIONAL \c {} block if the
 * group is set to be optional.
 */
void
PatternGroup::setOptional(bool optional)
{
	data()->m_optional = optional;
}

/*!
 * Returns the list of patterns in this group.
 *
 * Note that this list is actually a tree, since it can contain PatternGroup
 * instances, that contains restrictions themselves.
 */
QList<PatternBase>
PatternGroup::patterns() const
{
	return data()->m_patterns;
}

/*!
 * Sets the list of patterns of this group.
 *
 * The existing patterns will get replaced by the patterns in the list.
 */
void
PatternGroup::setPatterns(const QList<PatternBase> &patterns)
{
	data()->m_patterns = patterns;
}

/*!
 * Adds a new pattern to the group.
 *
 * You will probably use this function to a nested PatternGroup to the group.
 *
 * The user keeps ownership of the pattern and must free it.
 */
void
PatternGroup::addPattern(const PatternBase &pattern)
{
	data()->m_patterns.append(pattern);
}

/*!
 * Adds a new triple to the group.
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
PatternGroup::addPattern(const Value &s, const Value &p, const Value &o)
{
	Pattern pattern(s, p, o);
	addPattern(pattern);
}

/*!
 * Adds a triple chain to the group.
 *
 * A triple chain allows you to easily generate triples for complex graph
 * patterns. Say that you want to select the phone numbers of all the
 * affiliations of a contact, you would create the a \c ValueChain with the
 * three predicate \c nco:hasAffiliation, \c nco:hasPhoneNumber,
 * \c nco:phoneNumber and two variables, for example \c ?c and \c ?phoneNumber.
 *
 * Passed those parameters, the function would generate the following
 * pattern:
 * \code
 * ?_c nco:hasAffiliation ?tmp0 . ?tmp0 nco:hasPhoneNumber ?tmp1 . ?tmp1 nco:phoneNumber ?_phoneNumber
 * \endcode
 * Note that user variable are always prefixed with \c _ in queries to avoid
 * name clashes with variables generated automatically.
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
PatternGroup::addPattern(const Value &s, const ValueChain &chain, const Value &o)
{
	ValueChain varChain;
	Value lhs, rhs;

	if (chain.empty()) {
		return;
	}

	if (chain.size() == 1) {
		addPattern(s, chain.first(), o);
		return;
	}

	lhs = s;

	for(ValueChain::ConstIterator i = chain.constBegin(); i != chain.constEnd(); ++i) {
		if (i+1 == chain.constEnd()) {
			rhs = o;
		} else {
			rhs = getTmpVar();
		}

		addPattern(lhs, *i, rhs);
		lhs = rhs;
	}
}

/*!
 * Returns the current group's filter, or an invalid Filter if there is none.
 */
Filter
PatternGroup::filter() const
{
	return data()->m_filter;
}

/*!
 * Sets a filter on the group.
 */
void
PatternGroup::setFilter(const Filter &filter)
{
	data()->m_filter = filter;
}

const PatternGroupData*
PatternGroup::data() const
{
	return static_cast<const PatternGroupData *>(d.constData());
}

PatternGroupData*
PatternGroup::data()
{
	return static_cast<PatternGroupData *>(d.data());
}


Variable
PatternGroup::getTmpVar()
{
	static const QString tmpVarNameTemplate = QString::fromLatin1("tmp%1");
	GeneratedVariable tmp(tmpVarNameTemplate.arg(data()->m_tmpVarIndex++));

	return tmp;
}
