/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "predicatefunction.h"

#include "function_p.h"

CUBI_BEGIN_NAMESPACE

static const QChar DefaultPredicateFunctionValueSeparator = QLatin1Char(',');
static const QChar DefaultPredicateFunctionGraphSeparator = QLatin1Char(';');

/*! \cond DOCUMENT_DATA_CLASSES */
class PredicateFunctionData : public FunctionData
{

public:
	PredicateFunctionData();
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual bool isValid() const;

	virtual Function::Type functionType() const;

	ResourceValue m_predicate;
	PredicateFunction::Options m_options;
	QChar m_valueSeparator;
	QChar m_graphSeparator;
};
/*! \endcond */

PredicateFunctionData::PredicateFunctionData()
	: m_options(PredicateFunction::NoOptions)
	, m_valueSeparator(DefaultPredicateFunctionValueSeparator)
	, m_graphSeparator(DefaultPredicateFunctionGraphSeparator)
{
}

ValueData*
PredicateFunctionData::clone()
{
	return new PredicateFunctionData(*this);
}

bool
PredicateFunctionData::operator==(const ValueData *other) const
{
	if (other->type() != Value::TypeFunction) {
		return false;
	}

	if (static_cast<const FunctionData*>(other)->functionType() !=
	    Function::TypePredicateFunction) {
		return false;
	}

	return m_predicate ==
	       static_cast<const PredicateFunctionData*>(other)->m_predicate;
}

QString
PredicateFunctionData::sparql() const
{
	static const QString sparqlTemplate = QString::fromLatin1("%1(%2)");
	static const QString sparqlTemplateWithValueSeparator = QString::fromLatin1("%1(%2, \"\\u%3\")");
	static const QString sparqlTemplateWithValueAndGraphSeparators = QString::fromLatin1("%1(%2, \"\\u%3\", \"\\u%4\")");
	
	if (m_parameters.empty()) {
		qWarning("Trying to apply a PredicateFunction without parameters set");
		return QString();
	}

	if (PredicateFunction::FetchGraph == m_options) {
		return sparqlTemplateWithValueAndGraphSeparators.
			arg(m_predicate.sparql(), m_parameters.first().sparql()).
			arg(m_valueSeparator.unicode(),4,16,QLatin1Char('0')).
			arg(m_graphSeparator.unicode(),4,16,QLatin1Char('0'));
	} else {
		if (m_valueSeparator != DefaultPredicateFunctionValueSeparator) {
			return sparqlTemplateWithValueSeparator.
				arg(m_predicate.sparql(), m_parameters.first().sparql()).
				arg(m_valueSeparator.unicode(),4,16,QLatin1Char('0'));
		} else {
			return sparqlTemplate.
				arg(m_predicate.sparql(), m_parameters.first().sparql());
		}
	}
}

bool
PredicateFunctionData::isValid() const
{
	return m_predicate.isValid();
}

Function::Type
PredicateFunctionData::functionType() const
{
	return Function::TypePredicateFunction;
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class PredicateFunction
 * Represents a predicate function.
 *
 * Predicate functions can only have one parameter.
 *
 * Note, predicate functions are not in the official Sparql specification, but
 * they are supported by some RDF databases like Tracker.
 */

/*!
 * Constructs a prefix function from its name.
 */
PredicateFunction::PredicateFunction(const ResourceValue &predicate)
	: Function(new PredicateFunctionData)
{
	data()->m_predicate = predicate;
}

/*!
 * Sets the predicate of this function.
 */
void
PredicateFunction::setPredicate(const ResourceValue &predicate)
{
	data()->m_predicate = predicate;
}

/*!
 * Returns the function's predicate.
 */
ResourceValue
PredicateFunction::predicate() const
{
	return data()->m_predicate;
}

/*!
 * Sets the separator char for multiple property values.
 * Default is @code','@endcode.
 */
void
PredicateFunction::setValueSeparator(const QChar &separator)
{
	data()->m_valueSeparator = separator;
}

/*!
 * Returns the separator char for multiple property values.
 */
QChar
PredicateFunction::valueSeparator() const
{
	return data()->m_valueSeparator;
}

/*!
 * Sets the separator char for the graph.
 * Also set the option property to @c FetchGraph.
 */
void
PredicateFunction::setGraphSeparator(const QChar &separator)
{
	data()->m_graphSeparator = separator;
	data()->m_options = FetchGraph;
}

/*!
 * Returns the separator char for the graph.
 */
QChar
PredicateFunction::graphSeparator() const
{
	return data()->m_graphSeparator;
}

/*!
 * Sets the options of the function.
 * Default is @c NoOptions.
 */
void
PredicateFunction::setOptions(Options options)
{
	data()->m_options = options;
}

/*!
 * Returns the options of the function.
 */
PredicateFunction::Options
PredicateFunction::options() const
{
	return data()->m_options;
}

/*!
 * Applies the function to \p v.
 *
 * Returns a new PredicateFunction with the parameters set (see Function
 * documentation).
 */
PredicateFunction
PredicateFunction::apply(const Value &v) const
{
	PredicateFunction f = *this;

	ValueChain parameters;
	parameters << v;
	f.data()->m_parameters = parameters;

	return f;
}

const PredicateFunctionData*
PredicateFunction::data() const
{
	return static_cast<const PredicateFunctionData *>(d.constData());
}

PredicateFunctionData*
PredicateFunction::data()
{
	return static_cast<PredicateFunctionData *>(d.data());
}
