/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "prefixfunction.h"

#include "function_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class PrefixFunctionData : public FunctionData
{

public:
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual bool isValid() const;

	virtual Function::Type functionType() const;

	QString m_name;

};
/*! \endcond */

ValueData*
PrefixFunctionData::clone()
{
	return new PrefixFunctionData(*this);
}

bool
PrefixFunctionData::operator==(const ValueData *other) const
{
	const PrefixFunctionData *otherData = 0;

	if (other->type() != Value::TypeFunction) {
		return false;
	}

	if (static_cast<const FunctionData*>(other)->functionType() !=
	    Function::TypePrefixFunction) {
		return false;
	}

	otherData = static_cast<const PrefixFunctionData*>(other);

	return m_name == otherData->m_name &&
	       m_parameters == otherData->m_parameters;
}

QString
PrefixFunctionData::sparql() const
{
	static const QString sparqlTemplate = QString::fromLatin1("%1(%2)");
	QStringList params;

	foreach(const Value &v, m_parameters) {
		params.append(v.sparql());
	}

	return sparqlTemplate.arg(m_name, params.join(", "));
}

bool
PrefixFunctionData::isValid() const
{
	return not m_name.isNull();
}

Function::Type
PrefixFunctionData::functionType() const
{
	return Function::TypePrefixFunction;
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class PrefixFunction
 * Represents a prefix function.
 *
 * Prefix functions can have an arbitrary number of parameters.
 */

/*!
 * Construts a prefix function from its name.
 */
PrefixFunction::PrefixFunction(const QString &name)
	: Function(new PrefixFunctionData)
{
	data()->m_name = name;
}

/*!
 * Sets the name of the function.
 */
void
PrefixFunction::setName(const QString &name)
{
	data()->m_name = name;
}

/*!
 * Returns the name of the function.
 */
QString
PrefixFunction::name() const
{
	return data()->m_name;
}

/*!
 * Applies the given \p values as parameters of the function.
 *
 * Returns a new PrefixFunction with the parameters set (see Function
 * documentation).
 */
PrefixFunction
PrefixFunction::apply(const ValueChain &values) const
{
	PrefixFunction f = *this;

	f.data()->m_parameters = values;

	return f;
}

/*!
 * Applies the function to \p v1.
 *
 * Returns a new PrefixFunction with the parameters set (see Function
 * documentation).
 */
PrefixFunction
PrefixFunction::apply(const Value &v1) const
{
	ValueChain params;
	params << v1;
	return apply(params);
}

/*!
 * Applies the function to \p v1 and \p v2.
 *
 * Returns a new PrefixFunction with the parameters set (see Function
 * documentation).
 */
PrefixFunction
PrefixFunction::apply(const Value &v1, const Value &v2) const
{
	ValueChain params;
	params << v1;
	params << v2;
	return apply(params);
}

/*!
 * Applies the function to \p v1, \p v2 and \p v3.
 *
 * Returns a new PrefixFunction with the parameters set (see Function
 * documentation).
 */
PrefixFunction
PrefixFunction::apply(const Value &v1, const Value &v2, const Value &v3) const
{
	ValueChain params;
	params << v1;
	params << v2;
	params << v3;
	return apply(params);
}

const PrefixFunctionData*
PrefixFunction::data() const
{
	return static_cast<const PrefixFunctionData *>(d.constData());
}

PrefixFunctionData*
PrefixFunction::data()
{
	return static_cast<PrefixFunctionData *>(d.data());
}
