/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "projection.h"

#include "select.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class ProjectionData : public QSharedData
{

public:
	ProjectionData();

	Variable variable;
	Function function;
	Select select;
	LiteralValue literalValue;
	Projection::Type type;
};
/*! \endcond */

ProjectionData::ProjectionData()
	: QSharedData()
{
	variable = Variable::invalid();
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Projection
 * Represents a projection in a Select.
 *
 * A projection in a Select statement can be a variable, a predicate function
 * or a nested Select. The Projection class holds one of those three classes.
 */

/*!
 * \enum Projection::Type
 * Describes the type of the projection.
 */

/*!
 * Creates a projection for a variable.
 *
 * You will likely never use this function, but rather get Projection instances
 * from Select::projections().
 */
Projection::Projection(const Variable &variable)
	: d(new ProjectionData)
{
	d->variable = variable;
	d->type = Projection::VariableProjection;
}

/*!
 * Creates a projection for a predicate function.
 *
 * You will likely never use this function, but rather get Projection instances
 * from Select::projections().
 */
Projection::Projection(const Function &function, const Variable &alias)
	: d(new ProjectionData)
{
	d->function = function;
	d->type = Projection::FunctionProjection;
	d->variable = alias;
}

/*!
 * Creates a projection for a nested Select.
 *
 * You will likely never use this function, but rather get Projection instances
 * from Select::projections().
 */
Projection::Projection(const Select &select, const Variable &alias)
	: d(new ProjectionData)
{
	d->select = select;
	d->type = Projection::SelectProjection;
	d->variable = alias;
}

/*!
 * Creates a projection for a literal value
 *
 * You will likely never use this function, but rather get Projection instances
 * from Select::projections().
 */
Projection::Projection(const LiteralValue &literalValue, const Variable &alias)
	: d(new ProjectionData)
{
	d->literalValue = literalValue;
	d->type = Projection::LiteralProjection;
	d->variable = alias;
}

Projection::Projection(const Projection &other)
	: d(other.d)
{
}

Projection&
Projection::operator=(const Projection &other)
{
	return d = other.d, *this;
}

bool
Projection::operator==(const Projection &other) const
{
	return d->variable == other.d->variable &&
	       d->literalValue == other.d->literalValue &&
	       d->function == other.d->function &&
	       d->select == other.d->select &&
	       d->type == other.d->type;
}

bool
Projection::operator!=(const Projection &other) const
{
	return not (*this == other);
}

Projection::~Projection()
{
}

/*!
 * Returns the type of the projection.
 *
 * \sa Projection::Type
 */
Projection::Type
Projection::type() const
{
	return d->type;
}

/*!
 * Returns the variable of the projection.
 *
 * If called on a projection which is not of type
 * Projection::VariableProjection, will print an error message and return
 * an invalid Variable
 */
Variable
Projection::variable() const
{
	if (d->type != Projection::VariableProjection) {
		qWarning("Trying to get variable value from non variable typed projection");
		return Variable::invalid(); // enforce empty name, thus invalid Variable
	}

	return d->variable;
}

/*!
 * Returns the function of the projection.
 *
 * If called on a projection which is not of type
 * Projection::FunctionProjection, will print an error message and return
 * an invalid Function
 */
Function
Projection::function() const
{
	if (d->type != Projection::FunctionProjection) {
		qWarning("Trying to get function value from non function typed projection");
		return Function();
	}

	return d->function;
}

/*!
 * Returns the select of the projection.
 *
 * If called on a projection which is not of type
 * Projection::SelectProjection, will print an error message and return
 * an empty Select. // FIXME: how to best test for empty state?
 */
Select
Projection::select() const
{
	if (d->type != Projection::SelectProjection) {
		qWarning("Trying to get select value from non select typed projection");
		return Select();
	}

	return d->select;
}

/*!
 * Returns the literal value of the projection.
 *
 * If called on a projection which is not of type
 * Projection::LiteralProjection, will print an error message and return
 * an invalid LiteralValue().
 */
LiteralValue
Projection::literalValue() const
{
	if (d->type != Projection::LiteralProjection) {
		qWarning("Trying to get literal value from non literal typed projection");
		return LiteralValue();
	}

	return d->literalValue;
}

/*!
 * Returns the alias of the projection.
 *
 * If called on a projection which is not of type
 * Projection::SelectProjection or Projection::FunctionProjection, will print
 * an error message and return an invalid Variable
 */
Variable
Projection::alias() const
{
	if (d->type == Projection::VariableProjection) {
		qWarning("Trying to get alias for a variable");
		return Variable::invalid(); // enforce empty name, thus invalid Variable
	}

	return d->variable;
}

/*!
 * Returns the Sparql representation of the projection.
 */
QString
Projection::sparql(Options::SparqlOptions options) const
{
	static const QString aliasTemplate = QString::fromLatin1("%1 AS %2");
	static const QString selectTemplate = QString::fromLatin1("(%1)");

	switch(d->type) {
		case VariableProjection:
			return d->variable.sparql();
			break;
		case FunctionProjection:
			if (d->variable.isValid()) {
				return aliasTemplate.arg(d->function.sparql(), d->variable.sparql());
			} else {
				return d->function.sparql();
			}
			break;
		case SelectProjection:
			if (d->variable.isValid()) {
				return aliasTemplate.arg(selectTemplate.arg(d->select.sparql(options)), d->variable.sparql());
			} else {
				return selectTemplate.arg(d->select.sparql(options));
			}
			break;
		case LiteralProjection:
			if (d->variable.isValid()) {
				return aliasTemplate.arg(d->literalValue.sparql(), d->variable.sparql());
			} else {
				return d->literalValue.sparql();
			}
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unknown projection type");
			return QString();
	}
}

