/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "resourcevalue.h"

#include "utils.h"
#include "value_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class ResourceValueData : public ValueData
{

public:
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	virtual Value::Type type() const;
	virtual bool isValid() const;

	ResourceValue::Type m_type;
	QString m_iri;
	QString m_name;
};
/*! \endcond */

ValueData*
ResourceValueData::clone()
{
	return new ResourceValueData(*this);
}

bool
ResourceValueData::operator==(const ValueData *other) const
{
	const ResourceValueData *otherData = 0;

	if (other->type() != Value::TypeResourceValue) {
		return false;
	}

	otherData = static_cast<const ResourceValueData*>(other);

	if (m_type != otherData->m_type) {
		return false;
	}

	switch(m_type) {
		case ResourceValue::Invalid:
			return true;
		case ResourceValue::Iri:
			return m_iri == otherData->m_iri;
		case ResourceValue::PrefixedName:
			return m_name == otherData->m_name;
		default:
			qFatal("%s: Unhandled type %d", Q_FUNC_INFO, m_type);
			return false;
	}
}

QString
ResourceValueData::sparql() const
{
	if (not isValid()) {
		return QString();
	}

	static const QString iriTemplate = QString::fromLatin1("<%1>");

	switch(m_type) {
		case ResourceValue::Invalid:
			return QString();
		case ResourceValue::Iri:
			return iriTemplate.arg(Utils::escapeIri(m_iri));
		case ResourceValue::PrefixedName:
			return m_name;
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			return QString();
	}
}

Value::Type
ResourceValueData::type() const
{
	return Value::TypeResourceValue;
}

bool
ResourceValueData::isValid() const
{
	switch(m_type) {
		case ResourceValue::Invalid:
			return false;
		case ResourceValue::Iri:
			return not m_iri.isEmpty();
		case ResourceValue::PrefixedName:
			return not m_name.isEmpty();
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			return false;
	}
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class ResourceValue
 * Models a Sparql IRI reference.
 */

/*!
 * Constructs an empty, invalid ResourceValue.
 */
ResourceValue::ResourceValue()
	: Value(new ResourceValueData)
{
	data()->m_type = Invalid;
}

/*!
 * Constructs a ResourceValue from its IRI.
 */
ResourceValue::ResourceValue(const QUrl &iri)
	: Value(new ResourceValueData)
{
	data()->m_iri = iri.toString();
	data()->m_type = Iri;
}

/*!
 * Constructs a ResourceValue from its prefixed name.
 */
ResourceValue::ResourceValue(const QString &name, ResourceValue::Type type)
	: Value(new ResourceValueData)
{
	switch (type) {
		case Iri:
			data()->m_iri = name;
			data()->m_type = Iri;
			break;
		case PrefixedName:
			data()->m_name = name;
			data()->m_type = PrefixedName;
			break;
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			break;
	}
}

bool
ResourceValue::operator==(const ResourceValue &other) const
{
	if (other.resourceType() != data()->m_type) {
		return false;
	}

	switch(data()->m_type) {
		case ResourceValue::Invalid:
			return true;
		case ResourceValue::Iri:
			return data()->m_iri == other.data()->m_iri;
		case ResourceValue::PrefixedName:
			return data()->m_name == other.data()->m_name;
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			return false;
	}

}

bool
ResourceValue::operator!=(const ResourceValue &other) const
{
	return not (*this == other);
}

/*!
 * Returns the type of the value.
 *
 * The type can be one of
 * \li \c ResourceValue::Invalid an invalid (unset) value
 * \li \c ResourceValue::Iri a complete IRI reference (like <tt>&lt;iriref&gt;</tt>)
 * \li \c ResourceValue::PrefixedName a prefixed name (like <tt>ns:Name</tt>)
 */
ResourceValue::Type
ResourceValue::resourceType() const
{
	return data()->m_type;
}

/*!
 * Sets the IRI of the ResourceValue from a QUrl.
 *
 * This function also sets the type of the value to ResourceValue::Iri.
 */
void
ResourceValue::setIri(const QUrl &iri)
{
	data()->m_iri = iri.toString();
	data()->m_type = Iri;
}

/*!
 * Sets the IRI of the ResourceValue from a QString.
 *
 * This function also sets the type of the value to ResourceValue::Iri.
 */
void
ResourceValue::setIri(const QString &iri)
{
	data()->m_iri = iri;
	data()->m_type = Iri;
}


/*!
 * Returns the IRI of the ResourceValue.
 *
 * If called on a value of type ResourceValue::PrefixedName, returns a
 * converted value.
 */
QString
ResourceValue::iri() const
{
	switch(data()->m_type) {
		case ResourceValue::Invalid:
			return QString();
		case ResourceValue::Iri:
			return data()->m_iri;
		case ResourceValue::PrefixedName:
			return data()->m_name;
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			return QString();
	}
}

/*!
 * Sets the name of the ResourceValueData.
 *
 * This function also sets the type of the value to ResourceValue::PrefixedName.
 */
void
ResourceValue::setName(const QString &name)
{
	data()->m_name = name;
	data()->m_type = PrefixedName;
}

/*!
 * Returns the name of the ResourceValue.
 *
 * If called on a value of type ResourceValue::Iri, returns a converted value.
 */
QString
ResourceValue::name() const
{
	switch(data()->m_type) {
		case ResourceValue::Invalid:
			return QString();
		case ResourceValue::Iri:
			return data()->m_iri;
		case ResourceValue::PrefixedName:
			return data()->m_name;
		default:
			qFatal("%s: %s", Q_FUNC_INFO, "Unhandled type");
			return QString();
	}
}

const ResourceValueData*
ResourceValue::data() const
{
	return static_cast<const ResourceValueData *>(d.constData());
}

ResourceValueData*
ResourceValue::data()
{
	return static_cast<ResourceValueData *>(d.data());
}
