/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "select.h"

#include "patterngroup.h"
#include "utils.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class SelectData : public QSharedData
{

public:
	SelectData();
	~SelectData();

	QList<Projection> projections;
	PatternGroup group;
	long limit;
	long offset;
	Value orderBy;

};
/*! \endcond */

SelectData::SelectData()
	: QSharedData()
	, limit(-1)
	, offset(-1)
{
}

SelectData::~SelectData()
{
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Select
 * Models a SELECT statement in SPARQL.
 *
 * That class can be used for both "normal" and nested SELECTs.
 *
 * A Select statement has projections and restrictions. The projections are the
 * variables you want to get in the result set, and the restrictions are the
 * graph patterns that must be satisfied by the triples returned in the result
 * set.
 *
 * Let's consider the following query:
 * \code
 * SELECT ?c WHERE {?c a nco:PersonContact}
 * \endcode
 * In this query, The \c ?c variable is a projection, and the only retriction is
 * the graph pattern in the curly braces.
 */

/*!
 * Constructs an empty SELECT statement.
 *
 * The returned statement has no projections, no restrictions, no results
 * limit and no results offset.
 */
Select::Select()
	: d(new SelectData)
{
}

Select::Select(const Select &other)
	: d(other.d)
{
}

Select&
Select::operator=(const Select &other)
{
	return d = other.d, *this;
}

bool
Select::operator==(const Select &other) const
{
	return d->projections == other.d->projections &&
	       d->group == other.d->group &&
	       d->limit == other.d->limit &&
	       d->offset == other.d->offset &&
	       d->orderBy == other.d->orderBy;
}

bool
Select::operator!=(const Select &other) const
{
	return not (*this == other);
}

Select::~Select()
{
}

/*!
 * Adds a new projection to the statement's projections.
 */
void
Select::addProjection(const Projection &projection)
{
	d->projections.append(projection);
}

/*!
 * Adds a new variable to the statement's projections.
 */
void
Select::addProjection(const Variable &variable)
{
	d->projections.append(Projection(variable));
}

/*!
 * Adds a new predicate function to the statement's projections.
 *
 * The function must have previously been applied to another projection using
 * Function::apply(), this will not be checked by the library.
 *
 * An optional alias can be defined for the projection using the \p alias
 * parameter.
 */
void
Select::addProjection(const Function &function, const Variable &alias)
{
	d->projections.append(Projection(function, alias));
}

/*!
 * Adds an inner select to the statement's projections.
 *
 * The inner select must have one single projection, and return 0 or 1 results
 * for the query to work.
 *
 * An optional alias can be defined for the projection using the \p alias
 * parameter.
 */
void
Select::addProjection(const Select &select, const Variable &alias)
{
	d->projections.append(Projection(select, alias));
}

/*!
 * Adds a literal value to the statement's projections.
 *
 * This adds a constant column to the results.
 */
void
Select::addProjection(const LiteralValue &literalValue)
{
	d->projections.append(Projection(literalValue));
}

/*!
 * Adds a new restriction to the statement.
 *
 * You will probably use this function to add a PatternGroup to the statement's
 * restrictions.
 *
 * The user keeps ownership of the pattern and must free it.
 */
void
Select::addRestriction(const PatternBase &pattern)
{
	d->group.addPattern(pattern);
}

/*!
 * Adds a new triple to the statement's restrictions.
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
Select::addRestriction(const Value &s, const Value &p, const Value &o)
{
	d->group.addPattern(s, p, o);
}

/*!
 * Adds a triple chain to the statement's restrictions.
 *
 * \sa PatternGroup::addPattern(const Value &s, const ValueChain &chain, const Value &o)
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
Select::addRestriction(const Value &s, const ValueChain &chain, const Value &o)
{
	d->group.addPattern(s, chain, o);
}

/*!
 * Returns the current statement's restriction filter, or an invalid Filter if there is
 * none.
 */
Filter
Select::filter() const
{
	return d->group.filter();
}

/*!
 * Sets a global filter on the statement's restrictions.
 *
 * More specific filters applying only to a subset of the restrictions can be
 * done setting filters on PatternGroup instances and adding those instances to
 * the restrictions.
 */
void
Select::setFilter(const Filter &filter)
{
	d->group.setFilter(filter);
}

/*!
 * Sets the maximum number of results to return.
 *
 * -1 can be passed to disable this feature.
 */
void
Select::setLimit(long limit)
{
	d->limit = limit;
}

/*!
 * Set number of results to skip at the beginning of the result set.
 *
 * -1 can be passed to disable this feature.
 */
void
Select::setOffset(long offset)
{
	d->offset = offset;
}

/*!
 * Sets the expression used to order the results in the result set.
 */
void
Select::setOrderBy(const Value &order)
{
	d->orderBy = order;
}

/*!
 * Returns @c true if there are no projections, no restrictions and no filter set.
 */
bool
Select::isEmpty() const
{
	return d->projections.isEmpty() &&
	       d->group.patterns().isEmpty() &&
	       (not d->group.filter().isValid());
}

/*!
 * Returns all the projections of this statement.
 */
QList<Projection>
Select::projections() const
{
	return d->projections;
}

/*!
 * Returns all the restrictions of this statement.
 *
 * Note that this list is actually a tree, since it can contain PatternGroup
 * instances, that contains restrictions themselves.
 */
QList<PatternBase>
Select::restrictions() const
{
	return d->group.patterns();
}

/*!
 * Returns the current results limit of this statement, or -1 if there is no
 * limit set.
 */
long
Select::limit() const
{
	return d->limit;
}

/*!
 * Returns the current results offset of this statement, or -1 if there is no
 * offset set.
 */
long
Select::offset() const
{
	return d->offset;
}

/*!
 * Returns the expression used to sort the results, or an invalid Value if none
 * is set.
 */
Value
Select::orderBy() const
{
	return d->orderBy;
}

/*!
 * Returns the Sparql query corresponding to this statement.
 */
QString
Select::sparql(Options::SparqlOptions options) const
{
	static const QString baseTemplate = QString::fromLatin1("SELECT %1 WHERE {%2}%3");
	static const QString baseTemplatePretty = QString::fromLatin1("SELECT\n%1\nWHERE\n{\n%2\n}%3");
	static const QString limitTemplate = QString::fromLatin1(" LIMIT %1");
	static const QString offsetTemplate = QString::fromLatin1(" OFFSET %1");
	static const QString orderByTemplate = QString::fromLatin1(" ORDER BY %1");
	QStringList projectionNames;
	QString modifiers;

	foreach(const Projection &projection, d->projections) {
		projectionNames.append(projection.sparql(options));
	}

	if (d->limit != -1) {
		modifiers.append(limitTemplate.arg(d->limit));
	}

	if (d->offset != -1) {
		modifiers.append(offsetTemplate.arg(d->offset));
	}

	if (d->orderBy.isValid()) {
		modifiers.append(orderByTemplate.arg(d->orderBy.sparql()));
	}

	if (options & Options::PrettyPrint) {
		return Utils::prettyPrint(baseTemplatePretty.arg(projectionNames.join("\n"),
		                                                 d->group.sparql(options),
		                                                 modifiers));
	} else {
		return  baseTemplate.arg(projectionNames.join(" "), d->group.sparql(options), modifiers);
	}
}
