/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "union.h"

#include "patternbase_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class UnionData : public PatternBaseData
{

public:
	virtual PatternBaseData * clone();
	virtual bool operator==(const PatternBaseData *other) const;
	virtual bool isUnion() const;
	virtual PatternBase::Type type() const;
	virtual bool isValid() const;
	virtual QString sparql(Options::SparqlOptions options) const;

	PatternBase m_lhs, m_rhs;
};
/*! \endcond */

PatternBaseData*
UnionData::clone()
{
	return new UnionData(*this);
}

bool
UnionData::operator==(const PatternBaseData *other) const
{
	const UnionData *otherData = 0;

	if (type() != other->type()) {
		return false;
	}

	otherData = static_cast<const UnionData*>(other);

	return m_lhs == otherData->m_lhs &&
	       m_rhs == otherData->m_rhs;
}

bool
UnionData::isUnion() const
{
	return true;
}

PatternBase::Type
UnionData::type() const
{
	return PatternBase::TypeUnion;
}

bool
UnionData::isValid() const
{
	return m_lhs.isValid() && m_rhs.isValid();
}

QString
UnionData::sparql(Options::SparqlOptions options) const
{
	static const QString unionTemplate = QString::fromLatin1("{%1} UNION {%2}");
	static const QString unionTemplatePretty = QString::fromLatin1("{\n%1\n} UNION {\n%2\n}");

	return (options & Options::PrettyPrint) ? unionTemplatePretty.arg(m_lhs.sparql(options), m_rhs.sparql(options))
	                                        : unionTemplate.arg(m_lhs.sparql(options), m_rhs.sparql(options));
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class Union
 * Models the Sparql UNION keyword.
 */

/*
 * Constructs a Union with invalid lhs and rhs.
 */
Union::Union()
	: PatternBase(new UnionData)
{
}

/*!
 * Constructs the union of two graph patterns.
 */
Union::Union(const PatternBase &lhs, const PatternBase &rhs)
	: PatternBase(new UnionData)
{
	data()->m_lhs = lhs;
	data()->m_rhs = rhs;
}

/*!
 * Sets the Union's lhs.
 */
void
Union::setLhs(const PatternBase &lhs)
{
	data()->m_lhs = lhs;
}

/*!
 * Returns the left hand side operator of the union.
 */
PatternBase
Union::lhs() const
{
	return data()->m_lhs;
}

/*!
 * Sets the Union's rhs.
 */
void
Union::setRhs(const PatternBase &rhs)
{
	data()->m_rhs = rhs;
}

/*!
 * Returns the right hand side operator of the union.
 */
PatternBase
Union::rhs() const
{
	return data()->m_rhs;
}

const UnionData*
Union::data() const
{
	return static_cast<const UnionData *>(d.constData());
}

UnionData*
Union::data()
{
	return static_cast<UnionData *>(d.data());
}
