/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "updatebase.h"

#include "updatebase_p.h"

CUBI_USE_NAMESPACE

UpdateBaseData::~UpdateBaseData()
{
}

bool
UpdateBaseData::operator==(const UpdateBaseData *other) const
{
	return type()       == other->type() &&
	       graph        == other->graph &&
	       data         == other->data &&
	       restrictions == other->restrictions;
}

template<>
UpdateBaseData *QSharedDataPointer<UpdateBaseData>::clone()
{
	return d->clone();
}

///////////////////////////////////////////////////////////////////////////////

/*!
 * \class UpdateBase
 * Common class for update statements.
 *
 * The UpdateBase class is the base for the Sparql statements used to modify a
 * graph, that is Insert and Delete. You will never use this class directly,
 * only its subclasses.
 */

UpdateBase::UpdateBase(const UpdateBase &other)
	: d(other.d)
{
}

UpdateBase&
UpdateBase::operator=(const UpdateBase &other)
{
	return d = other.d, *this;
}

bool
UpdateBase::operator==(const UpdateBase &other) const
{
	return *d.constData() == other.d;
}

bool
UpdateBase::operator!=(const UpdateBase &other) const
{
	return not (*this == other);
}

UpdateBase::UpdateBase(UpdateBaseData *data)
	: d(data)
{
}

UpdateBase::~UpdateBase()
{
}

/*!
 * Retrurns the type of the UpdateBase.
 */
UpdateBase::Type
UpdateBase::type() const
{
	return d->type();
}

/*!
 * Adds a graph pattern to the data part of this statement.
 *
 * You will most likely use this function to insert PatternGroup instances into
 * the data part.
 *
 * The user keeps the ownership of the pattern and must free it.
 */
void
UpdateBase::addData(const PatternBase &pattern)
{
	d->data.addPattern(pattern);
}

/*!
 * Adds a triple to the data part of this statement.
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
UpdateBase::addData(const Value &s, const Value &p, const Value &o)
{
	d->data.addPattern(s, p, o);
}

/*!
 * Returns all the patterns in the data part of this statement.
 *
 * Note that this list is actually a tree, since it can contain PatternGroup
 * instances, that contains restrictions themselves.
 */
QList<PatternBase>
UpdateBase::data() const
{
	return d->data.patterns();
}

/*!
 * Adds a graph pattern to the restrictions of this statement.
 *
 * You will most likely use this function to insert PatternGroup instances into
 * the restrictions.
 *
 * The user keeps the ownership of the pattern and must free it.
 */
void
UpdateBase::addRestriction(const PatternBase &pattern)
{
	d->restrictions.addPattern(pattern);
}

/*!
 * Adds a triple to the restrictions of this statement.
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
UpdateBase::addRestriction(const Value &s, const Value &p, const Value &o)
{
	d->restrictions.addPattern(s, p, o);
}

/*!
 * Adds a triple chain to the statement's restrictions.
 *
 * \sa PatternGroup::addPattern(const Value *s, const ValueChain &chain, const Value *o)
 *
 * The user keeps the ownership of the variables and must free them.
 */
void
UpdateBase::addRestriction(const Value &s, const ValueChain &chain, const Value &o)
{
	d->restrictions.addPattern(s, chain, o);
}

/*!
 * Returns all the patterns in the restrictions of this statement.
 *
 * Note that this list is actually a tree, since it can contain PatternGroup
 * instances, that contains restrictions themselves.
 */
QList<PatternBase>
UpdateBase::restrictions() const
{
	return d->restrictions.patterns();
}

/*!
 * Returns the current statement's restriction filter, or NULL if there is
 * none.
 */
Filter
UpdateBase::filter() const
{
	return d->restrictions.filter();
}

/*!
 * Sets a global filter on the statement's restrictions.
 *
 * More specific filters applying only to a subset of the restrictions can be
 * done setting filters on PatternGroup instances and adding those instances to
 * the restrictions.
 */
void
UpdateBase::setFilter(const Filter &filter)
{
	d->restrictions.setFilter(filter);
}

QString
UpdateBase::sparql(Options::SparqlOptions options) const
{
	return d->sparql(options);
}

