/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "utils.h"

CUBI_USE_NAMESPACE

QString
Utils::prettyPrint(const QString &query)
{
	return escape(indent(query));
}

QString
Utils::indent(const QString &query)
{
	static const int indentSize = 2; // Number of spaces to indent with
	int indentLevel = 0;
	QStringList tokens = query.split('\n');
	QStringList resultLines;

	foreach(const QString &str, tokens) {
		QString line;

		if (str.trimmed().startsWith("}")) {
			indentLevel --;
		}

		indentLevel = qMax(indentLevel, 0);

		for (int i = 0; i < (indentSize * indentLevel); ++i) {
			line += ' ';
		}

		line += str;
		resultLines.append(line);

		if (str.trimmed().endsWith("{")) {
			indentLevel ++;
		}
	}

	return resultLines.join("\n");
}

QString
Utils::escape(const QString &query)
{
	static const QString escapeTemplate = "\\u%1";

	QString result;
	int head = 0;

	for (int tail = 0; tail < query.length(); ++tail) {
		const QChar ch = query[tail];

		if (not ch.isPrint() && ch != '\n') {
			result += query.mid(head, tail - head);
			result += escapeTemplate.arg(ch.unicode(), 4, 16, QLatin1Char('0'));
			head = tail + 1;
		}
	}

	result += query.mid(head);

	return result;
}

static inline QString
percentEncode(QChar c)
{
	static const QString shortTemplate = QString::fromLatin1("%%1");
	static const QString longTemplate = QString::fromLatin1("%%1%%2");

	if (c.row() == 0) {
		return shortTemplate.arg((uint)c.cell(), 0, 16);
	}

	return longTemplate.arg((uint)c.row(), 0, 16).arg((uint)c.cell(), 0, 16);
}

static inline bool
isForbiddenSparqlCharacter(QChar c)
{
	const ushort code = c.unicode();

	// See http://www.w3.org/TR/rdf-sparql-query/#rIRI_REF
	switch (code) {
		case '\\':
		case '<':
		case '>':
		case '"':
		case '{':
		case '}':
		case '|':
		case '^':
		case '`':
			return true;
		default:
			break;
	}

	return (code <= 0x20);
}

QString
Utils::escapeIri(const QString &iri)
{
	int nEscaped = 0;

	// Count how many escaping we have to do
	for (int i = 0; i < iri.size(); ++i) {
		if (isForbiddenSparqlCharacter(iri.at(i))) {
			nEscaped ++;
		}
	}

	if (nEscaped == 0) {
		return iri;
	}

	QString escaped;
	// Original size + %XX%YY percent encoding * number of escaped chars
	escaped.reserve(iri.size() + 5 * nEscaped);

	for (int i = 0; i < iri.size(); ++i) {
		const QChar c = iri.at(i);

		if (not isForbiddenSparqlCharacter(c)) {
			escaped.append(c);
			continue;
		}

		const QString percentEncoded = percentEncode(c);
		escaped.append(percentEncoded);
	}

	return escaped;
}

#define IN_RANGE(value,start,end) \
	(value >= start && value <= end)

// See http://www.w3.org/TR/rdf-sparql-query/#rPN_LOCAL & associated rules

static bool
isPN_CHARS_BASE(ushort code)
{
	return (IN_RANGE(code, 'a', 'z')
	     || IN_RANGE(code, 'A', 'Z')
	     || IN_RANGE(code, 0xC0, 0xD6)
	     || IN_RANGE(code, 0xD8, 0xF6)
	     || IN_RANGE(code, 0xF8, 0x2FF)
	     || IN_RANGE(code, 0x370, 0x37D)
	     || IN_RANGE(code, 0x37F, 0x1FFF)
	     || IN_RANGE(code, 0x2070, 0x218F)
	     || IN_RANGE(code, 0x2C00, 0x2FEF)
	     || IN_RANGE(code, 0x3001, 0xD7FF)
	     || IN_RANGE(code, 0xF900, 0xFDCF)
	     || IN_RANGE(code, 0xFDF0, 0xFFFD));
	// There is also 10000 - EFFFF but that does not fit in the ushort
}
static bool
isPN_CHARS_U(ushort code)
{
	return (isPN_CHARS_BASE(code)
	     || code == '_');
}

static bool
isAllowedBlankCharacter(const QChar &c, bool first)
{
	const ushort code = c.unicode();

	if (IN_RANGE(code, '0', '9')
	 || isPN_CHARS_U(code)) {
		return true;
	}

	// Rest of the characters are only allowed if it's not the first character
	if (first) {
		return false;
	}

	// PN_CHARS
	return ((code == '-')
	     || (code == 0xB7)
	     || IN_RANGE(code, 0x300, 0x36F)
	     || IN_RANGE(code, 0x203F, 0x2040));
}

static bool
isAllowedVariableCharacter(const QChar &c, bool first)
{
	const ushort code = c.unicode();

	if (IN_RANGE(code, '0', '9')
	 || isPN_CHARS_U(code)) {
		return true;
	}

	// Rest of the characters are only allowed if it's not the first character
	if (first) {
		return false;
	}

	return ((code == 0xB7)
	     || IN_RANGE(code, 0x300, 0x36F)
	     || IN_RANGE(code, 0x203F, 0x2040));
}

typedef bool(*charFilterFunc)(const QChar &, bool);

static QString
escapeName(const QString &name, charFilterFunc filter)
{
	// This function only replaces forbidden chars by '_', as there is no escaping
	// character for names. This implies a risk of "collision", ie. the escaped
	// names for both "test:" and "test!" will be "test_". A warning is printed
	// in this case.
	static const QChar replacementChar = QLatin1Char('_');

	bool didEscape = false;
	QString escaped = name;

	for (int i = 0; i < escaped.size(); ++i) {
		if (not filter(escaped.at(i), (i == 0))) {
			didEscape = true;
			escaped[i] = replacementChar;
		}
	}

	if (didEscape) {
		qWarning("%s: Replaced invalid name '%s' with '%s'",
		         Q_FUNC_INFO,
		         qPrintable(name),
		         qPrintable(escaped));
	}

	return escaped;
}

QString
Utils::escapeBlankName(const QString &name)
{
	return escapeName(name, &isAllowedBlankCharacter);
}

QString
Utils::escapeVariableName(const QString &name)
{
	return escapeName(name, &isAllowedVariableCharacter);
}
