/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "valuelist.h"

#include "value_p.h"

CUBI_BEGIN_NAMESPACE

/*! \cond DOCUMENT_DATA_CLASSES */
class ValueListData : public ValueData
{

public:
	virtual ValueData * clone();
	virtual bool operator==(const ValueData *other) const;
	virtual QString sparql() const;
	Value::Type type() const;
	virtual bool isValid() const;

	QList<Value> m_values;
};
/*! \endcond */

ValueData*
ValueListData::clone()
{
	return new ValueListData(*this);
}

bool
ValueListData::operator==(const ValueData *other) const
{
	if (other->type() != Value::TypeValueList) {
		return false;
	}

	return m_values == static_cast<const ValueListData*>(other)->m_values;
}

QString
ValueListData::sparql() const
{
	static const QString listTemplate = QString::fromLatin1("(%1)");
	QStringList sparqlValues;

	foreach(const Value &v, m_values) {
		sparqlValues << v.sparql();
	}

	return listTemplate.arg(sparqlValues.join(", "));
}

Value::Type
ValueListData::type() const
{
	return Value::TypeValueList;
}

bool
ValueListData::isValid() const
{
	return true;
}

CUBI_END_NAMESPACE

///////////////////////////////////////////////////////////////////////////////

CUBI_USE_NAMESPACE

/*!
 * \class ValueList
 * Models a Sparql list.
 *
 * This class is different from a simple QList<Value> because it can generate
 * the Sparql representation of the list.
 *
 * Note that the list does not take ownership of the values, the user must free
 * them.
 */

/*!
 * Constructs an empty list.
 */
ValueList::ValueList()
	: Value(new ValueListData)
{
}

/*!
 * Returns the values in this list.
 */
QList<Value>
ValueList::values() const
{
	return data()->m_values;
}

/*!
 * Replaces the values of the list with the values in \p values.
 */
void
ValueList::setValues(const QList<Value> &values)
{
	data()->m_values = values;
}

/*!
 * Adds a new value to the list.
 */
void
ValueList::addValue(const Value &value)
{
	data()->m_values.append(value);
}

const ValueListData*
ValueList::data() const
{
	return static_cast<const ValueListData *>(d.constData());
}

ValueListData*
ValueList::data()
{
	return static_cast<ValueListData *>(d.data());
}

