/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "variable.h"

#include "utils.h"
#include "variable_p.h"

CUBI_USE_NAMESPACE

VariableData::VariableData()
	: ValueData()
	, m_auto(false)
{
}

ValueData*
VariableData::clone()
{
	return new VariableData(*this);
}

bool
VariableData::operator==(const ValueData *other) const
{
	const VariableData *otherData;

	if (other->type() != Value::TypeVariable) {
		return false;
	}

	otherData = static_cast<const VariableData*>(other);

	return m_name == otherData->m_name &&
	       m_auto == otherData->m_auto;
}

QString
VariableData::sparql() const
{
	static const QString nameTemplate = QString::fromLatin1("?_%1");
	static const QString autoNameTemplate = QString::fromLatin1("?%1");

	const QString escapedName = Utils::escapeVariableName(m_name);

	return m_auto ? autoNameTemplate.arg(escapedName) : nameTemplate.arg(escapedName);
}

Value::Type
VariableData::type() const
{
	return Value::TypeVariable;
}

bool
VariableData::isValid() const
{
	return not m_name.isNull();
}

///////////////////////////////////////////////////////////////////////////////

/*!
 * \class Variable
 * Models a Sparql variable.
 */

/*!
 * Constructs a variable with an automatically assigned name.
 *
 * \sa Variable::create()
 */
Variable::Variable()
	: Value(new VariableData)
{
	static int varCounter = 0;
	static const QString nameTemplate = QString::fromLatin1("auto%1");
	data()->m_name = nameTemplate.arg(varCounter++);
	data()->m_auto = true;
}

/*!
 * Constructs a varible from its \p name.
 */
Variable::Variable(const QString &name)
	: Value(new VariableData)
{
	data()->m_name = name;
}

Variable::Variable(VariableData *data)
	: Value(data)
{
}

/*!
 * Returns the variable's name.
 *
 * Note the the original name and the name in the generated Sparql differ, this
 * is because Cubi prefixes user variable names with an _, in order to avoid
 * name clashes with the variables it generates.
 */
QString
Variable::name() const
{
	return data()->m_name;
}

/*!
 * Sets the name of the variable.
 */
void
Variable::setName(const QString &name)
{
	data()->m_name = name;
	data()->m_auto = false;
}

/*!
 * Constructs a variable with an automatically assigned name (as Variable() does)
 *
 * \sa Variable()
 */
Variable
Variable::create()
{
	return Variable();
}

/*!
 * Constructs an invalid Variable
 */
Variable
Variable::invalid()
{
	static const Variable invalidVariable = Variable(QString());
	return invalidVariable;
}

const VariableData*
Variable::data() const
{
	return static_cast<const VariableData *>(d.constData());
}

VariableData*
Variable::data()
{
	return static_cast<VariableData *>(d.data());
}
