/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "ut_data.h"

#include <cubi.h>

#include <QTest>

#define MAKEVAR(name) Variable name(QString::fromLatin1(#name));


CUBI_USE_NAMESPACE

void
UtData::testBlankValue()
{
	BlankValue v1;

	QVERIFY(v1.type() == Value::TypeBlankValue);
	QVERIFY(v1 != Value());
	QVERIFY(v1.isValid());
	QCOMPARE(v1.sparql(), QString::fromLatin1("_:auto0"));

	BlankValue v2(v1);
	QVERIFY(v1 == v2);

	v2 = BlankValue(QString());
	QVERIFY(not v2.isValid());
	QVERIFY(v1 != v2);

	v2.setName(QString::fromLatin1("test"));
	QVERIFY(v2.isValid());
	QCOMPARE(v2.name(), QString::fromLatin1("test"));
	QCOMPARE(v2.sparql(), QString::fromLatin1("_:_test"));

	v1 = v2;
	v1.setName(QString::fromLatin1("other"));
	QCOMPARE(v1.name(), QString::fromLatin1("other"));
	QCOMPARE(v2.name(), QString::fromLatin1("test"));

	v1.setName(QString::fromLatin1("hello:world!"));
	QCOMPARE(v1.sparql(), QString::fromLatin1("_:_hello_world_"));
}

void
UtData::testCompositionalSelect()
{
	Select mainSelect;
	Select innerSelect;
	MAKEVAR(a);
	MAKEVAR(b);
	MAKEVAR(c);

	innerSelect.addProjection(a);
	innerSelect.addRestriction(a, b, c);
	mainSelect.addProjection(a);
	mainSelect.addRestriction(CompositionalSelect(innerSelect));

	QCOMPARE(mainSelect.sparql(), QString::fromLatin1("SELECT ?_a WHERE {{SELECT ?_a WHERE {?_a ?_b ?_c}}}"));

	CompositionalSelect s1(innerSelect);
	QVERIFY(s1.isCompositionalSelect());
	QVERIFY(s1.isValid());
	QCOMPARE(s1.type(), PatternBase::TypeCompositionalSelect);
	QCOMPARE(s1.select(), innerSelect);
	QVERIFY(s1 == CompositionalSelect(innerSelect));
	QVERIFY(s1 != CompositionalSelect(mainSelect));
	QVERIFY(s1 != PatternGroup());

	CompositionalSelect s2 = s1;
	QVERIFY(s1 == s2);
}

void
UtData::testExists()
{
	ResourceValue r1(QString::fromLatin1("test:foo")), r2(QUrl::fromEncoded("contact:5"));
	LiteralValue v1(QString::fromLatin1("foobar")), v2(QVariant(35));

	Exists e1;
	QVERIFY(e1.isValid());
	QCOMPARE(e1.type(), PatternBase::TypeExists);
	QVERIFY(e1.isExists());

	PatternGroup p;
	QVERIFY(e1 != p);

	e1.addPattern(r2, r1, v1);

	Exists e2 = e1;
	QVERIFY(e2 == e1);

	e2 = Exists(e1);
	QVERIFY(e2 == e1);

	e2.addPattern(r2, r1, v2);
	QVERIFY(e1 != e2);
}

void
UtData::testFilter()	
{
	ResourceValue r1(QString::fromLatin1("test:foo"), ResourceValue::PrefixedName), r2(QUrl::fromEncoded("contact:5"));
	LiteralValue v1(QString::fromLatin1("foobar")), v2(QVariant(35)), v3(QVariant(false));
	PrefixFunction fntest(QString::fromLatin1("fn:test"));
	MAKEVAR(var);

	Filter f1;

	QVERIFY(not f1.isValid());
	QCOMPARE(f1.filterType(), Filter::InvalidFilter);
	QCOMPARE(f1.sparql(), QString());
	QVERIFY(f1.exists() == Exists());
	QVERIFY(f1.function() == Function());
	QVERIFY(f1.literalValue() == Value());
	QVERIFY(f1.select() == Select());

	Exists exists;
	exists.addPattern(r2, r1, v1);

	f1 = Filter(exists);
	QCOMPARE(f1.type(), Value::TypeFilter);
	QVERIFY(f1.isValid());
	QCOMPARE(f1.filterType(), Filter::ExistsFilter);
	QVERIFY(f1.exists() == exists);
	QCOMPARE(f1.sparql(), QString::fromLatin1("EXISTS {<contact:5> test:foo \"foobar\"}"));

	Filter f2(f1);
	QVERIFY(f1 == f2);

	f2 = Filter(fntest.apply(r2, v2));
	QVERIFY(f2.isValid());
	QVERIFY(f2 != f1);
	QCOMPARE(f2.filterType(), Filter::FunctionFilter);
	QVERIFY(f2.function() == fntest.apply(r2, v2));
	QCOMPARE(f2.sparql(), QString::fromLatin1("fn:test(<contact:5>, 35)"));

	f2 = Filter(v3);
	QVERIFY(f2.isValid());
	QCOMPARE(f2.filterType(), Filter::LiteralFilter);
	QVERIFY(f2.literalValue() == v3);
	QCOMPARE(f2.sparql(), QString::fromLatin1("false"));

	Select s;
	s.addProjection(var);
	s.addRestriction(var, r1, v1);
	f2 = Filter(s);
	QVERIFY(f2.isValid());
	QCOMPARE(f2.filterType(), Filter::SelectFilter);
	QVERIFY(f2.select() == s);
	QCOMPARE(f2.sparql(), QString::fromLatin1("(SELECT ?_var WHERE {?_var test:foo \"foobar\"})"));
}

void
UtData::testFunction()
{
	Function f1;
	QVERIFY(not f1.isValid());
	QCOMPARE(f1.type(), Value::TypeFunction);
	QCOMPARE(f1.functionType(), Function::TypeInvalid);
	QCOMPARE(f1.sparql(), QString());

	LiteralValue v1(QVariant(35));
	ValueChain values;

	Function f2 = f1;
	QVERIFY(f2 == f1);
	values << v1;
	f2.setParameters(values);
	QVERIFY(f2 != f1);
	QVERIFY(not f2.isValid());
	QCOMPARE(f2.parameters(), values);
}

void
UtData::testGraph()
{
	ResourceValue resource(QUrl::fromEncoded("foo:bar"));
	MAKEVAR(var);
	Graph g1(resource);

	QVERIFY(g1.isGraph());

	QVERIFY(g1.isValid());
	QVERIFY(g1 == Graph(resource));
	QVERIFY(g1.iri() == resource);
	QCOMPARE(g1.variable(), Variable::invalid());
	QCOMPARE(g1.type(), PatternBase::TypeGraph);
	QCOMPARE(g1.graphType(), Graph::IriGraph);

	Graph g2 = g1;
	QVERIFY(g1 == g2);
	QVERIFY(g2.isValid());
	QVERIFY(g2.iri() == resource);
	QCOMPARE(g2.type(), PatternBase::TypeGraph);
	QCOMPARE(g2.graphType(), Graph::IriGraph);

	g2.setIri(ResourceValue());
	QVERIFY(g1 != g2);
	QVERIFY(not g2.isValid());
	QCOMPARE(g2.iri(), ResourceValue());

	g2.setVariable(var);
	QVERIFY(g2 == Graph(var));
	QVERIFY(g1 != g2);
	QVERIFY(g2.isValid());
	QCOMPARE(g2.iri(), ResourceValue());
	QCOMPARE(g2.variable(), var);
	QCOMPARE(g2.graphType(), Graph::VariableGraph);

	g2.setVariable(Variable::invalid());
	QVERIFY(not g2.isValid());
	QCOMPARE(g2.variable(), Variable::invalid());
}

void
UtData::testLiteral_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	LiteralValue v = LiteralValue(false);

	// Booleans

	QTest::newRow("Bool false")
		<< v.sparql()
		<< QString::fromLatin1("false");

	v = LiteralValue(true);

	QTest::newRow("Bool true")
		<< v.sparql()
		<< QString::fromLatin1("true");

	// TODO Date & DateTime
	// TODO Double Int LongLong UInt ULongLong

	// Strings and chars
	v = LiteralValue(QChar::fromLatin1('c'));
	QTest::newRow("Char")
		<< v.sparql()
		<< QString::fromLatin1("\"c\"");

	v = LiteralValue(QLatin1String("Hello world"));

	QTest::newRow("Simple string")
		<< v.sparql()
		<< QString::fromLatin1("\"Hello world\"");

	v = LiteralValue(QLatin1String("\t\n\r\"\\"));

	QTest::newRow("Escape sequences")
		<< v.sparql()
		<< QString::fromLatin1("\"\\t\\n\\r\\\"\\\\\"");

	v = LiteralValue(QString::fromUtf8("\u0141\u00D8\u0159\u015D"));

	QTest::newRow("Non-ASCII characters")
		<< v.sparql()
		<< QString::fromUtf8("\"\u0141\u00d8\u0159\u015d\"");

	// TODO List
	// TODO StringList

	v = LiteralValue(QUrl(QString::fromLatin1("http://hello.world.org")));
	QTest::newRow("Url")
		<< v.sparql()
		<< LiteralValue(QString::fromLatin1("http://hello.world.org")).sparql();
}

void
UtData::testLiteral()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

void
UtData::testPatternGroup_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	PatternGroup group;

	QTest::newRow("Empty group")
		<< group.sparql()
		<< QString::fromLatin1("");

	Value subject, predicate, object;
	group.addPattern(subject, predicate, object);

	QTest::newRow("Group with invalid pattern")
		<< group.sparql()
		<< QString::fromLatin1("  ");

	// TODO complete test and check coverage
}

void
UtData::testPatternGroup()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

void
UtData::testPredicateFunction()
{
	ResourceValue invalidValue;
	PredicateFunction invalidFunction(invalidValue);

	QVERIFY(not invalidFunction.isValid());
	QCOMPARE(invalidFunction.type(), Value::TypeFunction);
	QCOMPARE(invalidFunction.functionType(), Function::TypePredicateFunction);

	PredicateFunction f1(invalidFunction);
	QVERIFY(f1 == invalidFunction);
	QVERIFY(f1 != LiteralValue(3.14));
	QVERIFY(f1 != PrefixFunction(QString::fromLatin1("foo")));

	ResourceValue aPredicate = ResourceValue(QString::fromLatin1("some-resource"));
	f1.setPredicate(aPredicate);
	QVERIFY(f1.isValid());
	QVERIFY(f1 != invalidFunction);
	QVERIFY(f1 == PredicateFunction(aPredicate));
	QCOMPARE(f1.predicate(), aPredicate);
	QCOMPARE(f1.options(), PredicateFunction::NoOptions);

	MAKEVAR(param);

	QCOMPARE(f1.sparql(), QString());
	QCOMPARE(f1.apply(param).sparql(), QString::fromLatin1("<some-resource>(?_param)"));

	f1.setPredicate(ResourceValue(QString::fromLatin1("func"), ResourceValue::PrefixedName));
	QCOMPARE(f1.apply(param).sparql(), QString::fromLatin1("func(?_param)"));

	f1.setOptions(PredicateFunction::FetchGraph);
	QCOMPARE(f1.options(), PredicateFunction::FetchGraph);
	QCOMPARE(f1.apply(param).sparql(), QString::fromLatin1("func(?_param, \"\\u002c\", \"\\u003b\")"));

	f1.setValueSeparator(QChar::fromLatin1('a'));
	f1.setGraphSeparator(QChar::fromLatin1('b'));
	QCOMPARE(f1.valueSeparator(), QChar::fromLatin1('a'));
	QCOMPARE(f1.graphSeparator(), QChar::fromLatin1('b'));
	QCOMPARE(f1.apply(param).sparql(), QString::fromLatin1("func(?_param, \"\\u0061\", \"\\u0062\")"));

	f1.setOptions(PredicateFunction::NoOptions);
	QCOMPARE(f1.apply(param).sparql(), QString::fromLatin1("func(?_param, \"\\u0061\")"));
}

void
UtData::testPrefixFunction()
{
	PrefixFunction f1(QString::fromLatin1(""));
	f1.setName(QString());
	QVERIFY(not f1.isValid());
	QCOMPARE(f1.name(), QString());
	QCOMPARE(f1.functionType(), Function::TypePrefixFunction);

	PrefixFunction f2(f1);
	QVERIFY(f2 == f1);

	f2.setName(QString::fromLatin1("fn:test"));
	QVERIFY(f2 != f1);
	QCOMPARE(f2.name(), QString::fromLatin1("fn:test"));

	LiteralValue v1(QVariant(27));
	QCOMPARE(f2.sparql(), QString::fromLatin1("fn:test()"));
	QCOMPARE(f2.apply(v1).sparql(), QString::fromLatin1("fn:test(27)"));

	QVERIFY(f2 != v1);

	Function f;
	QVERIFY(f2 != f);
}

void
UtData::testProjection()
{
	// Variable projection
	MAKEVAR(var);

	Projection p1(var);
	QCOMPARE(p1.type(), Projection::VariableProjection);
	QCOMPARE(p1.variable(), var);
	QCOMPARE(p1.function(), Function());
	QCOMPARE(p1.select(), Select());
	QCOMPARE(p1.literalValue(), LiteralValue());
	QCOMPARE(p1.alias(), Variable::invalid());

	Projection p2 = p1;
	QVERIFY(p2 == p1);
	QVERIFY(p1 == Projection(var));

	QCOMPARE(Projection(var).sparql(), var.sparql());

	// Function projection
	PrefixFunction fn = PrefixFunction(QString::fromLatin1("fn:foo"));

	Projection p3(fn);
	QCOMPARE(p3.type(), Projection::FunctionProjection);
	QCOMPARE(p3.variable(), Variable::invalid());
	QVERIFY(p3.function() == fn);
	QCOMPARE(p3.select(), Select());
	QCOMPARE(p3.literalValue(), LiteralValue());
	QCOMPARE(p3.alias(), Variable::invalid());

	p2 = p3;
	QVERIFY(p3 == p2);
	QVERIFY(p3 != p1);
	QVERIFY(p3 == Projection(fn));

	QCOMPARE(Projection(fn.apply(var)).sparql(), fn.apply(var).sparql());

	// Function projection with alias
	Projection p4(fn, var);
	QCOMPARE(p4.type(), Projection::FunctionProjection);
	QCOMPARE(p4.variable(), Variable::invalid());
	QVERIFY(p4.function() == fn);
	QCOMPARE(p4.select(), Select());
	QCOMPARE(p4.literalValue(), LiteralValue());
	QCOMPARE(p4.alias(), var);

	p2 = p4;
	QVERIFY(p4 == p2);
	QVERIFY(p4 != p1);
	QVERIFY(p4 == Projection(fn, var));

	QCOMPARE(Projection(fn.apply(var), var).sparql(), QString::fromLatin1("fn:foo(?_var) AS ?_var"));

	// Select projection
	Select select;
	select.addProjection(var);

	Projection p5(select);
	QCOMPARE(p5.type(), Projection::SelectProjection);
	QCOMPARE(p5.variable(), Variable::invalid());
	QCOMPARE(p5.function(), Function());
	QCOMPARE(p5.select(), select);
	QCOMPARE(p5.literalValue(), LiteralValue());
	QCOMPARE(p5.alias(), Variable::invalid());

	p2 = p5;
	QVERIFY(p5 == p2);
	QVERIFY(p5 != p1);
	QVERIFY(p5 == Projection(select));

	QCOMPARE(p5.sparql(), QString::fromLatin1("(SELECT ?_var WHERE {})"));

	// Select projection with alias
	Projection p6(select, var);
	QCOMPARE(p6.type(), Projection::SelectProjection);
	QCOMPARE(p6.variable(), Variable::invalid());
	QCOMPARE(p6.function(), Function());
	QCOMPARE(p6.select(), select);
	QCOMPARE(p6.literalValue(), LiteralValue());
	QCOMPARE(p6.alias(), var);

	p2 = p6;
	QVERIFY(p6 == p2);
	QVERIFY(p6 != p1);
	QVERIFY(p6 == Projection(select, var));

	QCOMPARE(p6.sparql(), QString::fromLatin1("(SELECT ?_var WHERE {}) AS ?_var"));

	// Literal projection
	LiteralValue lvalue(3.14);

	Projection p7(lvalue);
	QCOMPARE(p7.type(), Projection::LiteralProjection);
	QCOMPARE(p7.variable(), Variable::invalid());
	QCOMPARE(p7.function(), Function());
	QCOMPARE(p7.select(), Select());
	QCOMPARE(p7.literalValue(), lvalue);
	QCOMPARE(p7.alias(), Variable::invalid());

	p2 = p7;
	QVERIFY(p7 == p2);
	QVERIFY(p7 != p1);
	QVERIFY(p7 == Projection(lvalue));

	QCOMPARE(p7.sparql(), QString::fromLatin1("3.14"));

	// Literal projection with alias
	Projection p8(lvalue, var);
	QCOMPARE(p8.type(), Projection::LiteralProjection);
	QCOMPARE(p8.variable(), Variable::invalid());
	QCOMPARE(p8.function(), Function());
	QCOMPARE(p8.select(), Select());
	QCOMPARE(p8.literalValue(), lvalue);
	QCOMPARE(p8.alias(), var);

	p2 = p8;
	QVERIFY(p8 == p2);
	QVERIFY(p8 != p1);
	QVERIFY(p8 == Projection(lvalue, var));

	QCOMPARE(p8.sparql(), QString::fromLatin1("3.14 AS ?_var"));
}

void
UtData::testInfixFunction()
{
	InfixFunction f1(QString::fromLatin1(""));
	f1.setName(QString());
	QVERIFY(not f1.isValid());
	QCOMPARE(f1.name(), QString());
	QCOMPARE(f1.functionType(), Function::TypeInfixFunction);

	InfixFunction f2(f1);
	QVERIFY(f2 == f1);

	f2.setName(QString::fromLatin1("%"));
	QVERIFY(f2 != f1);
	QCOMPARE(f2.name(), QString::fromLatin1("%"));

	LiteralValue v1(QVariant(27)), v2(QVariant(45));
	QCOMPARE(f2.sparql(), QString());
	QCOMPARE(f2.apply(v1).sparql(), QString::fromLatin1("27"));
	QCOMPARE(f2.apply(v1, v2).sparql(), QString::fromLatin1("(27 % 45)"));

	QVERIFY(f2 != v1);

	Function f;
	QVERIFY(f2 != f);
}

void
UtData::testResourceValue()
{
	ResourceValue r1;

	QCOMPARE(r1.type(), Value::TypeResourceValue);
	QCOMPARE(r1.resourceType(), ResourceValue::Invalid);
	QVERIFY(not r1.isValid());
	QVERIFY(r1 == ResourceValue());
	QCOMPARE(r1.sparql(), QString());

	r1 = ResourceValue(QUrl::fromEncoded(QByteArray("hello:world")));
	QCOMPARE(r1.resourceType(), ResourceValue::Iri);
	QVERIFY(r1.isValid());
	QCOMPARE(r1.iri(), QLatin1String(QByteArray("hello:world")));
	QCOMPARE(r1.sparql(), QString::fromLatin1("<hello:world>"));
	QVERIFY(r1 == ResourceValue(QUrl(QString::fromLatin1("hello:world"))));
	r1.setIri(QUrl());
	QVERIFY(not r1.isValid());
	QCOMPARE(r1.sparql(), QString());

	r1.setIri(QUrl(QString::fromLatin1("<hello:w\xF8rld>")));
	QCOMPARE(r1.sparql(), QString::fromLatin1("<%3chello:w\xF8rld%3e>"));

	ResourceValue r2(QString::fromLatin1("nao:hasTag"), ResourceValue::PrefixedName);
	QCOMPARE(r2.resourceType(), ResourceValue::PrefixedName);
	QVERIFY(r2.isValid());
	QCOMPARE(r2.name(), QString::fromLatin1("nao:hasTag"));
	r2.setName(QString::fromLatin1("nco:nickname"));
	QCOMPARE(r2.name(), QString::fromLatin1("nco:nickname"));
	QCOMPARE(r2.sparql(), QString::fromLatin1("nco:nickname"));
	r2.setName(QString());
	QVERIFY(not r2.isValid());
	QCOMPARE(r2.sparql(), QString());
}

void
UtData::testValueList()
{
	QStringList strList;
	strList << QString::fromLatin1("foo");
	strList << QString::fromLatin1("bar");
	strList << QString::fromLatin1("baz");

	// Verify that setValues and addValue results into the same ValueList
	ValueList valueList1;
	ValueList valueList2;
	QList<Value> listOfValues;
	foreach (const QString &str, strList) {
		Value v = LiteralValue(str);
		listOfValues << v;
		valueList1.addValue(v);
	}
	valueList2.setValues(listOfValues);
	QVERIFY(valueList1.isValid());
	QVERIFY(valueList2.isValid());
	QCOMPARE(valueList1.type(), Value::TypeValueList);
	QCOMPARE(valueList2.type(), Value::TypeValueList);
	QCOMPARE(valueList1, valueList2);

	// Verify that LiteralValue(QStringList) is the same as ValueList(QStringList)
	LiteralValue literalValue = LiteralValue(strList);
	QCOMPARE(literalValue.sparql(), valueList1.sparql());
}

int
main(int argc, char **argv)
{
	QCoreApplication app(argc, argv);

	UtData t;
	return QTest::qExec(&t, argc, argv);
}

#include "ut_data.moc"
