/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "ut_delete.h"

#include <cubi.h>

#include <QTest>

CUBI_USE_NAMESPACE

void
UtDelete::testDelete()
{
	ResourceValue r1(QString::fromLatin1("test:foo")), r2(QUrl::fromEncoded("contact:5"));
	LiteralValue v1(QString::fromLatin1("foobar")), v2(QVariant(35));

	Delete d1;
	QCOMPARE(d1.type(), UpdateBase::DeleteUpdate);
	d1.addData(r2, r1, v1);

	Delete d2 = d1;
	QVERIFY(d2 == d1);

	d2.addData(r2, r1, v2);
	QVERIFY(d2 != d1);
	QCOMPARE(d1.data().size(), 1);
	QCOMPARE(d2.data().size(), 2);
}

void
UtDelete::testQuery_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	ResourceValue aContactIri = ResourceValue(QUrl("contact:42"));
	ResourceValue rdfsType = ResourceValue(QUrl::fromEncoded("http://www.w3.org/2000/01/rdf-schema#type"));
	ResourceValue rdfsResource = ResourceValue(QUrl::fromEncoded("http://www.w3.org/2000/01/rdf-schema#Resource"));
	ResourceValue ncoContact = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"));
	ResourceValue ncoFullname = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname"));

	Delete d;
	Variable c = Variable("c");
	LiteralValue aFullName = LiteralValue(QVariant("John Zohrn"));
	PatternGroup whereClause;

	d.addData(c, rdfsType, ncoContact);
	whereClause.addPattern(c, ncoFullname, aFullName);
	d.addRestriction(whereClause);

	QTest::newRow("Delete a nco:Contact with WHERE clause")
		<< d.sparql()
		<< "DELETE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>} WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> \"John Zohrn\"}";

	///////////////////////////////////////////////////////////////////////////

	ResourceValue aGraph = ResourceValue(QUrl("mygraph"));
	d = Delete(aGraph);
	d.addData(c, rdfsType, ncoContact);
	d.addRestriction(whereClause);

	QTest::newRow("Delete a nco:Contact from graph and WHERE clause")
		<< d.sparql()
		<< "DELETE FROM <mygraph> {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>} WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> \"John Zohrn\"}";

	///////////////////////////////////////////////////////////////////////////

	Graph deleteGraph(aGraph);
	Graph whereGraph(aGraph);
	d = Delete();
	deleteGraph.addPattern(c, rdfsType, rdfsResource);
	whereGraph.addPattern(c, rdfsType, ncoContact);

	d.addData(deleteGraph);
	d.addRestriction(whereGraph);

	QTest::newRow("Delete a nco:Contact using GRAPH")
		<< d.sparql()
		<< "DELETE {"
		        "GRAPH <mygraph> {"
		        "?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.w3.org/2000/01/rdf-schema#Resource>"
		        "}"
		        "} WHERE {"
		        "GRAPH <mygraph> {"
		        "?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>"
		        "}"
		        "}";
}

void
UtDelete::testQuery()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

int
main(int argc, char **argv)
{
	QCoreApplication app(argc, argv);

	UtDelete t;
	return QTest::qExec(&t, argc, argv);
}

#include "ut_delete.moc"
