/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "ut_insert.h"

#include <cubi.h>

#include <QTest>

CUBI_USE_NAMESPACE

void
UtInsert::testQuery_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	ResourceValue aContactIri = ResourceValue(QUrl("contact:42"));
	ResourceValue rdfsType = ResourceValue(QUrl::fromEncoded("http://www.w3.org/2000/01/rdf-schema#type"));
	ResourceValue ncoContact = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"));
	ResourceValue ncoFullname = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname"));

	///////////////////////////////////////////////////////////////////////////

	Insert i;

	i.addData(aContactIri, rdfsType, ncoContact);

	QTest::newRow("Insert a nco:Contact")
		<< i.sparql()
		<< "INSERT {<contact:42> <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	ResourceValue aGraph = ResourceValue(QUrl("mygraph"));

	i = Insert(aGraph);
	i.addData(aContactIri, rdfsType, ncoContact);

	QTest::newRow("Insert a nco:Contact in graph")
		<< i.sparql()
		<< "INSERT INTO <mygraph> {<contact:42> <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	i = Insert(Insert::Replace);

	i.addData(aContactIri, rdfsType, ncoContact);

	QTest::newRow("Insert or replace a nco:Contact")
		<< i.sparql()
		<< "INSERT OR REPLACE {<contact:42> <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	i = Insert(Insert::Replace, aGraph);
	i.addData(aContactIri, rdfsType, ncoContact);

	QTest::newRow("Insert or replace a nco:Contact in graph")
		<< i.sparql()
		<< "INSERT OR REPLACE INTO <mygraph> {<contact:42> <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	LiteralValue aFullName = LiteralValue(QVariant("John Zohrn"));
	PatternGroup whereClause;

	i = Insert();
	i.addData(aContactIri, rdfsType, ncoContact);
	whereClause.addPattern(aContactIri, ncoFullname, aFullName);
	whereClause.setOptional(true);
	i.addRestriction(whereClause);

	QTest::newRow("Insert a nco:Contact with WHERE clause")
		<< i.sparql()
		<< "INSERT {<contact:42> <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>} WHERE {OPTIONAL {<contact:42> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> \"John Zohrn\"}}";

}

void
UtInsert::testQuery()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

int
main(int argc, char **argv)
{
	QCoreApplication app(argc, argv);

	UtInsert t;
	return QTest::qExec(&t, argc, argv);
}

#include "ut_insert.moc"
