/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "ut_optim.h"

#include <cubi.h>

#include <QTest>

#define MAKERESOURCE(name) ResourceValue name(QString::fromLatin1(#name), ResourceValue::PrefixedName);

CUBI_USE_NAMESPACE

void
UtOptim::testDuplicates_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	ResourceValue a("a", ResourceValue::PrefixedName),
	              b("b", ResourceValue::PrefixedName),
	              c("c", ResourceValue::PrefixedName),
	              d("d", ResourceValue::PrefixedName),
	              e("e", ResourceValue::PrefixedName),
	              f("f", ResourceValue::PrefixedName);

	Select s;

	s.addRestriction(a, b, c);
	s.addRestriction(a, b, c);
	s.addRestriction(b, c, d);
	
	QTest::newRow("Simple duplicate")
		<< s.sparql()
		<< QString::fromLatin1("SELECT  WHERE {a b c . b c d}");

	PatternGroup g;
	g.addPattern(c, d, e);
	g.addPattern(b, c, d);
	s.addRestriction(g);

	QTest::newRow("Duplicate with nested group")
		<< s.sparql()
		<< QString::fromLatin1("SELECT  WHERE {a b c . b c d . c d e}");

	g = PatternGroup();
	g.addPattern(a, b, c);
	g.setOptional(true);
	s.addRestriction(g);

	QTest::newRow("Check that optional does not get merged")
		<< s.sparql()
		<< QString::fromLatin1("SELECT  WHERE {a b c . b c d . c d e OPTIONAL {a b c}}");

	g = PatternGroup();
	PatternGroup o1, o2;
	o1.setOptional(true);
	o2.setOptional(true);
	o1.addPattern(a, b, c);
	o1.addPattern(c, d, e);
	o2.addPattern(a, b, c);
	o2.addPattern(c, e, f);
	g.addPattern(o1);
	g.addPattern(o2);


	QTest::newRow("Check that repeated statements in OPTIONALs get grouped")
		<< g.sparql()
		<< QString::fromLatin1("OPTIONAL {a b c OPTIONAL {c d e} OPTIONAL {c e f}}");

	Graph graph(ResourceValue(QUrl("mygraph")));
	graph.addPattern(a, b, c);
	g.addPattern(graph);

	QTest::newRow("Check that OPTIONAL cleaning preserves other items")
		<< g.sparql()
		<< QString::fromLatin1("GRAPH <mygraph> {a b c} OPTIONAL {a b c OPTIONAL {c d e} OPTIONAL {c e f}}");

	PatternGroup mainGroup;
	o1 = PatternGroup();
	o2 = PatternGroup();
	o1.setOptional(true);
	o1.addPattern(a, b, c);
	o2.setOptional(true);
	o2.addPattern(a, b, c);
	PatternGroup innerOpt1;
	innerOpt1.addPattern(c, d, e);
	innerOpt1.setOptional(true);
	o1.addPattern(innerOpt1);
	PatternGroup innerOpt2;
	innerOpt2.addPattern(c, e, f);
	innerOpt2.setOptional(true);
	o2.addPattern(innerOpt2);
	mainGroup.addPattern(o1);
	mainGroup.addPattern(o2);

	QTest::newRow("Check that repeated statements in OPTIONALs get grouped")
		<< mainGroup.sparql()
		<< QString::fromLatin1("OPTIONAL {a b c OPTIONAL {c d e} OPTIONAL {c e f}}");
}

void
UtOptim::testDuplicates()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

void
UtOptim::testGrouping_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	MAKERESOURCE(a);
	MAKERESOURCE(b);
	MAKERESOURCE(c);
	MAKERESOURCE(d);
	MAKERESOURCE(e);
	MAKERESOURCE(f);

	PatternGroup g;
	g.addPattern(a, b, c);
	g.addPattern(b, c, d);
	g.addPattern(a, c, e);
	g.addPattern(c, e, a);
	g.addPattern(b, c, e);

	QTest::newRow("Simple predicate object repetition + stability of subjects order")
		<< g.sparql(Options::DefaultSparqlOptions | Options::GroupPatterns)
		<< QString::fromLatin1("a b c ; c e . b c d, e . c e a");

	g = PatternGroup();

	QTest::newRow("Empty group")
		<< g.sparql(Options::DefaultSparqlOptions | Options::GroupPatterns)
		<< QString::fromLatin1("");

	g.addPattern(a, b, c);
	g.addPattern(a, b, d);
	g.addPattern(a, b, e);

	QTest::newRow("Simple value grouping")
		<< g.sparql(Options::DefaultSparqlOptions | Options::GroupPatterns)
		<< QString::fromLatin1("a b c, d, e");

	g = PatternGroup();
	g.addPattern(a, b, f);
	g.addPattern(a, c, f);
	g.addPattern(a, d, f);
	g.addPattern(a, e, f);

	QTest::newRow("Large subject grouping")
		<< g.sparql(Options::DefaultSparqlOptions | Options::GroupPatterns)
		<< QString::fromLatin1("a b f ; c f ; d f ; e f");
}

void
UtOptim::testGrouping()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

int
main(int argc, char **argv)
{
	QCoreApplication app(argc, argv);

	UtOptim t;
	return QTest::qExec(&t, argc, argv);
}

#include "ut_optim.moc"
