/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include "ut_select.h"

#include <cubi.h>

#include <QTest>

CUBI_USE_NAMESPACE

void
UtSelect::testSelect()
{
	LiteralValue v1(QString::fromLatin1("test")), v2(QVariant(42)), v3(QVariant(false));
	ResourceValue r1(QString::fromLatin1("test:foo"), ResourceValue::PrefixedName), r2(QUrl::fromEncoded("contact:5"));
	Select s1;

	QVERIFY(s1.isEmpty());

	Select s2;
	s1.addProjection(v1);
	s1.addRestriction(r2, r1, v2);
	s1.setLimit(10);
	s1.setOffset(10);
	s1.setOrderBy(r2);

	s2.addProjection(v1);
	s2.addRestriction(r2, r1, v2);
	s2.setLimit(10);
	s2.setOffset(10);
	s2.setOrderBy(r2);

	QVERIFY(s1 == s2);

	s2.setOffset(2);

	QVERIFY(s1 != s2);

	Select s3(s1);

	QVERIFY(s1 == s3);

	///////////////////////////////////////

	PrefixFunction f1(QString::fromLatin1("fn:test"));
	s1 = Select();
	QVERIFY(s1.isEmpty());

	s1.setFilter(Filter(f1.apply(r1, v1)));
	QCOMPARE(s1.filter().sparql(), QString::fromLatin1("fn:test(test:foo, \"test\")"));
}

void
UtSelect::testQuery_data()
{
	QTest::addColumn<QString>("generated");
	QTest::addColumn<QString>("expected");

	ResourceValue rdfsType = ResourceValue(QUrl::fromEncoded("http://www.w3.org/2000/01/rdf-schema#type"));
	ResourceValue ncoContact = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"));
	PredicateFunction FNcoFullname = PredicateFunction(ResourceValue("nco:fullname", ResourceValue::PrefixedName));
	ResourceValue ncoFullname = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname"));
	ResourceValue ncoHasPhoneNumber = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber"));
	ResourceValue ncoPhoneNumber = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber"));
	ResourceValue ncoHasAffiliation = ResourceValue(QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasAffiliation"));
	ValueChain phoneNumberChain, phoneNumberChainAffiliation;
	phoneNumberChain << ncoHasPhoneNumber << ncoPhoneNumber;
	phoneNumberChainAffiliation << ncoHasAffiliation << ncoHasPhoneNumber << ncoPhoneNumber;
	LiteralValue aPhoneNumber = LiteralValue(QVariant("00443326"));

	///////////////////////////////////////////////////////////////////////////

	Select s;
	Variable c = Variable("c");

	s.addProjection(c);
	s.addRestriction(c, rdfsType, ncoContact);

	QTest::newRow("Select a nco:Contact")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	PredicateFunction predicateFunction = FNcoFullname.apply(c);

	s.addProjection(predicateFunction);

	QTest::newRow("Select a nco:Contact and its nco:fullname")
		<< s.sparql()
		<< "SELECT ?_c nco:fullname(?_c) WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	Variable fullName = Variable("fullname");

	s = Select();
	s.addProjection(c);
	s.addProjection(predicateFunction, fullName);
	s.addRestriction(c, rdfsType, ncoContact);

	QTest::newRow("Select a nco:Contact and its nco:fullname aliased")
		<< s.sparql()
		<< "SELECT ?_c nco:fullname(?_c) AS ?_fullname WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	s = Select();
	s.addProjection(c);
	s.addRestriction(c, phoneNumberChain, aPhoneNumber);

	QTest::newRow("Select a nco:Contact and its phone number using ValueChain")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber> ?tmp0 . ?tmp0 <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber> \"00443326\"}";

	///////////////////////////////////////////////////////////////////////////

	s = Select();

	s.addProjection(c);
	s.addRestriction(c, rdfsType, ncoContact);
	s.setLimit(10);
	s.setOffset(5);

	QTest::newRow("Select a nco:Contact with LIMIT and OFFSET")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>} LIMIT 10 OFFSET 5";

	///////////////////////////////////////////////////////////////////////////

	// FIXME: Add test for nested groups
	PatternGroup group;
	s = Select();
	s.addProjection(c);
	group.setOptional(true);
	group.addPattern(c, phoneNumberChain, aPhoneNumber);
	s.addRestriction(group);

	QTest::newRow("Select a nco:Contact and its optional phone number using ValueChain")
		<< s.sparql()
		<< "SELECT ?_c WHERE {OPTIONAL {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber> ?tmp1 . ?tmp1 <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber> \"00443326\"}}";

	///////////////////////////////////////////////////////////////////////////

	group = PatternGroup();
	s = Select();
	s.addProjection(c);
	group.addPattern(c, ValueChain() << ncoFullname, fullName);
	s.addRestriction(group);

	QTest::newRow("Select a nco:Contact and its nco:fullname using a ValueChain with just one predicate")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> ?_fullname}";

	///////////////////////////////////////////////////////////////////////////

	s = Select();
	s.addProjection(c);

	PatternGroup g1, g2;
	g1.addPattern(c, phoneNumberChain, aPhoneNumber);
	g2.addPattern(c, phoneNumberChainAffiliation, aPhoneNumber);

	Union u = Union(g1, g2);
	s.addRestriction(u);

	QTest::newRow("Select a nco:Contact and its phone number using ValueChain with union")
		<< s.sparql()
		<< "SELECT ?_c WHERE {"
		                    "{?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber> ?tmp2 . ?tmp2 <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber> \"00443326\"}"
		                    " UNION "
		                    "{?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasAffiliation> ?tmp3 . ?tmp3 <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber> ?tmp4 . ?tmp4 <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber> \"00443326\"}}";

	///////////////////////////////////////////////////////////////////////////

	InfixFunction equal = InfixFunction("=");
	ResourceValue aIri = ResourceValue(QUrl("contact:42"));

	InfixFunction filter = equal.apply(c, aIri);

	s = Select();
	s.addProjection(c);
	s.addRestriction(c, rdfsType, ncoContact);
	s.setFilter(Filter(filter));

	QTest::newRow("Select a nco:Contact with simple filter")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact> FILTER((?_c = <contact:42>))}";

	///////////////////////////////////////////////////////////////////////////

	InfixFunction andFunc = InfixFunction("&&");
	InfixFunction lhs = filter;
	LiteralValue aFullName = LiteralValue(QVariant("John Zohrn"));
	InfixFunction rhs = equal.apply(fullName, aFullName);
	filter = andFunc.apply(lhs, rhs);

	s.addProjection(predicateFunction, fullName);
	s.setFilter(Filter(filter));

	QTest::newRow("Select a nco:Contact with complex filter")
		<< s.sparql()
		<< "SELECT ?_c nco:fullname(?_c) AS ?_fullname WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact> FILTER(((?_c = <contact:42>) && (?_fullname = \"John Zohrn\")))}";

	///////////////////////////////////////////////////////////////////////////

	LiteralValue anotherFullName = LiteralValue(QVariant("Miranda Alfazar"));
	ValueList filterValues;
	filterValues.addValue(aFullName);
	filterValues.addValue(anotherFullName);
	InfixFunction inFunc = InfixFunction("IN");
	filter = inFunc.apply(fullName, filterValues);
	s.setFilter(Filter(filter));


	QTest::newRow("Select a nco:Contact with complex filter (using IN)")
		<< s.sparql()
		<< "SELECT ?_c nco:fullname(?_c) AS ?_fullname WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact> FILTER((?_fullname IN (\"John Zohrn\", \"Miranda Alfazar\")))}";

	///////////////////////////////////////////////////////////////////////////

	Select innerSelect;

	s = Select();
	s.addProjection(c);

	innerSelect.addProjection(fullName);
	innerSelect.addRestriction(c, ncoFullname, fullName);
	s.addProjection(innerSelect);

	s.addRestriction(c, rdfsType, ncoContact);

	QTest::newRow("Select a nco:Contact with simple filter")
		<< s.sparql()
		<< "SELECT ?_c (SELECT ?_fullname WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> ?_fullname}) WHERE {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>}";

	///////////////////////////////////////////////////////////////////////////

	s = Select();
	s.addProjection(c);
	s.addRestriction(c, ncoFullname, fullName);
	s.setOrderBy(fullName);

	QTest::newRow("Select a nco:Contact and its nco:fullname with ordering")
		<< s.sparql()
		<< "SELECT ?_c WHERE {?_c <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname> ?_fullname} ORDER BY ?_fullname";

	///////////////////////////////////////////////////////////////////////////

	s = Select();
	s.addProjection(c);
	Exists exists;
	exists.addPattern(c, rdfsType, ncoContact);
	s.setFilter(Filter(exists));

	QTest::newRow("Select with EXISTS filter")
		<< s.sparql()
		<< "SELECT ?_c WHERE {FILTER(EXISTS {?_c <http://www.w3.org/2000/01/rdf-schema#type> <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact>})}";
}

void
UtSelect::testQuery()
{
	QFETCH(QString, generated);
	QFETCH(QString, expected);

	QCOMPARE(generated, expected);
}

int
main(int argc, char **argv)
{
	QCoreApplication app(argc, argv);

	UtSelect t;
	return QTest::qExec(&t, argc, argv);
}

#include "ut_select.moc"
