/****************************************************************************
 ** This file is part of Cubi, a Sparql query builder for the Qt framework
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (info@qt.nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
 **
 ** This file may be used under the terms of the GNU Lesser General Public
 ** License version 2.1 as published by the Free Software Foundation and
 ** appearing in the file LICENSE.LGPL included in the packaging of this file.
 ** Please review the following information to ensure the GNU Lesser General
 ** Public License version 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 **
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at info@qt.nokia.com.
 ****************************************************************************/

#include <QtSparql/QtSparql>

///////////////////////////////////////////////////////////////////////////////////////////////////

static const QLatin1String settingOutputDir("output-directory");
static const QLatin1String settingOverrides("overrides");

///////////////////////////////////////////////////////////////////////////////////////////////////

class Resource;
class Ontology;
class Class;
class Property;
class Literal;

///////////////////////////////////////////////////////////////////////////////////////////////////

class ResourceData : public QSharedData
{
    friend class Resource;

protected:
    ResourceData(const QString &iri)
        : m_iri(iri)
    {
    }

private:
    QString m_iri;
};

class Resource
{
public:
    Resource(const QString &iri)
        : d(new ResourceData(iri))
    {
    }

protected:
    Resource(ResourceData *data)
        : d(data)
    {
    }

public:
    const QString & iri() const
    {
        return data()->m_iri;
    }

protected:
    const ResourceData * data() const { return reinterpret_cast<const ResourceData *>(d.data()); }
    ResourceData * data() { return reinterpret_cast<ResourceData *>(d.data()); }

protected:
    QExplicitlySharedDataPointer<QSharedData> d;
};

///////////////////////////////////////////////////////////////////////////////////////////////////

class OntologyData : public ResourceData
{
    friend class Ontology;

protected:
    OntologyData(const QString &iri, const QString &prefix)
        : ResourceData(iri)
        , m_prefix(prefix)
    {
    }

private:
    QString m_prefix;
    QList<Class> m_classes;
    QList<Property> m_properties;
    QList<Literal> m_literals;
};

class Ontology : public Resource
{
public:
    Ontology(const QString &iri, const QString &prefix)
        : Resource(new OntologyData(iri, prefix))
    {
    }

public:
    const QString & prefix() const
    {
        return data()->m_prefix;
    }

    void setClasses(const QList<Class> &classes)
    {
        data()->m_classes = classes;
    }

    const QList<Class> & classes() const
    {
        return data()->m_classes;
    }

    void setProperties(const QList<Property> &properties)
    {
        data()->m_properties = properties;
    }

    const QList<Property> & properties() const
    {
        return data()->m_properties;
    }

    void setLiterals(const QList<Literal> &literals)
    {
        data()->m_literals = literals;
    }

    const QList<Literal> & literals() const
    {
        return data()->m_literals;
    }

public:
    QString alias(const QString &target) const
    {
        if (not target.startsWith(iri())) {
            return QString();
        }

        return prefix() + ":" + target.mid(iri().length());
    }

protected:
    const OntologyData * data() const { return reinterpret_cast<const OntologyData *>(d.data()); }
    OntologyData * data() { return reinterpret_cast<OntologyData *>(d.data()); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////

class ClassData : public ResourceData
{
    friend class Class;

protected:
    ClassData(const QString &iri, const Ontology &ontology)
        : ResourceData(iri)
        , m_name(iri.mid(ontology.iri().length()))
        , m_ontology(ontology)
    {
    }

private:
    QString m_name;
    const Ontology m_ontology;
};

class Class : public Resource
{
public:
    Class(const QString &iri, const Ontology &ontology)
        : Resource(new ClassData(iri, ontology))
    {
    }

public:
    const QString & name() const
    {
        return data()->m_name;
    }

    const Ontology & ontology() const
    {
        return data()->m_ontology;
    }

protected:
    const ClassData * data() const { return reinterpret_cast<const ClassData *>(d.data()); }
    ClassData * data() { return reinterpret_cast<ClassData *>(d.data()); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////

class PropertyData : public ResourceData
{
    friend class Property;

protected:
    PropertyData(const QString &iri, const Ontology &ontology,
                 const Class &domain, const Class &range, bool singleValued)
        : ResourceData(iri)
        , m_name(iri.mid(ontology.iri().length()))
        , m_ontology(ontology)
        , m_domain(domain)
        , m_range(range)
        , m_singleValued(singleValued)
    {
    }

private:
    QString m_name;
    Ontology m_ontology;
    Class m_domain;
    Class m_range;
    bool m_singleValued;
};

class Property : public Resource
{
public:
    Property(const QString &iri, const Ontology &ontology, const Class &domain, const Class &range, bool singleValued)
        : Resource(new PropertyData(iri, ontology, domain, range, singleValued))
    {
    }

public:
    const QString & name() const
    {
        return data()->m_name;
    }

    const Ontology & ontology() const
    {
        return data()->m_ontology;
    }

    const Class & domain() const
    {
        return data()->m_domain;
    }

    const Class & range() const
    {
        return data()->m_range;
    }

    bool singleValued() const
    {
        return data()->m_singleValued;
    }

protected:
    const PropertyData * data() const { return reinterpret_cast<const PropertyData *>(d.data()); }
    PropertyData * data() { return reinterpret_cast<PropertyData *>(d.data()); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////

class LiteralData : public ResourceData
{
    friend class Literal;

protected:
    LiteralData(const QString &iri, const Ontology &ontology, const Class &type)
        : ResourceData(iri)
        , m_name(iri.mid(type.ontology().iri().length()))
        , m_ontology(ontology)
        , m_type(type)
    {
    }

private:
    QString m_name;
    Ontology m_ontology;
    Class m_type;
};

class Literal : public Resource
{
public:
    Literal(const QString &iri, const Ontology &ontology, const Class &type)
        : Resource(new LiteralData(iri, ontology, type))
    {
    }

public:
    const QString & name() const
    {
        return data()->m_name;
    }

    const Ontology & ontology() const
    {
        return data()->m_ontology;
    }

    const Class & type() const
    {
        return data()->m_type;
    }

protected:
    const LiteralData * data() const { return reinterpret_cast<const LiteralData *>(d.data()); }
    LiteralData * data() { return reinterpret_cast<LiteralData *>(d.data()); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////

static QSparqlConnection sparql("QTRACKER_DIRECT");
static QMap<QString, Class> classesByIri;

static QTextStream &
err()
{
    static QTextStream stream(stderr);
    return stream << qAppName() << ": ";
}

static QSparqlResult *
query(const QString queryString)
{
    const QSparqlQuery query(queryString);
    QScopedPointer<QSparqlResult> result(sparql.syncExec(query));

    result->setParent(qApp);

    if (result->hasError()) {
        err() << result->lastError().message() << endl;
    }

    return result.take();
}

static QList<Literal>
readLiterals(const Class &type)
{
    QList<Literal> literals;

    const QString queryString =
            "SELECT ?l WHERE {"
            "  ?l a <" % type.iri() % "> ."
            "  FILTER(fn:starts-with(xsd:string(?l), \"" + type.ontology().iri() + "\")) ."
            "} ORDER BY tracker:id(?l)";

    for(QScopedPointer<QSparqlResult> result(query(queryString)); result->next(); ) {
        literals += Literal(result->value(0).toString(), type.ontology(), type);

        if ("tracker" == type.ontology().prefix()) {
            qDebug() << literals.count() << literals.last().name() << literals.last().iri() << type.iri();
        }
    }

    return literals;
}

static QList<Property>
readProperties(const Ontology &ontology, const QSettings &overrides)
{
    QList<Property> properties;

    const QString queryString =
            "SELECT ?p ?d ?r nrl:maxCardinality(?p) WHERE {"
            "  ?p a rdf:Property"
            "  ; rdfs:domain ?d"
            "  ; rdfs:range ?r"
            "  ."
            "  FILTER(fn:starts-with(xsd:string(?p), \"" + ontology.iri() + "\"))"
            "} ORDER BY tracker:id(?p)";

    for(QScopedPointer<QSparqlResult> result(query(queryString)); result->next(); ) {
        const QString propertyIri = result->value(0).toString();
        const QString key = ontology.alias(propertyIri);
        const QString domainIri = overrides.value(key + "/Domain", result->value(1)).toString();
        const QString rangeIri = overrides.value(key + "/Range", result->value(2)).toString();
        const bool singleValued = (not result->value(3).isNull() && (result->value(3).toLongLong() == 1));

        const QMap<QString, Class>::ConstIterator domain = classesByIri.find(domainIri);
        const QMap<QString, Class>::ConstIterator range = classesByIri.find(rangeIri);

        if (domain == classesByIri.constEnd()) {
            err() << propertyIri << ": Cannot find declaration of domain " << domainIri << endl;
            continue;
        }

        if (range == classesByIri.constEnd()) {
            err() << propertyIri << ": Cannot find declaration of range " << rangeIri << endl;
            continue;
        }

        properties += Property(propertyIri, ontology, *domain, *range, singleValued);
    }

    return properties;
}

static QList<Class>
readClasses(const Ontology &ontology)
{
    QList<Class> classes;

    const QString queryString =
            "SELECT ?c WHERE {"
            "  ?c a rdfs:Class ."
            "  FILTER(fn:starts-with(xsd:string(?c), \"" + ontology.iri() + "\")) ."
            "} ORDER BY tracker:id(?c)";

    for(QScopedPointer<QSparqlResult> result(query(queryString)); result->next(); ) {
        classes += Class(result->value(0).toString(), ontology);
        classesByIri.insert(classes.last().iri(), classes.last());
    }

    return classes;
}

static QList<Ontology>
readOntologies(const QSettings &overrides)
{
    const QString queryString =
            "SELECT ?o ?p WHERE {"
            "  ?o a tracker:Namespace ."
            "  ?o tracker:prefix ?p ."
            "}";

    QList<Ontology> ontologies;

    for(QScopedPointer<QSparqlResult> result(query(queryString)); result->next(); ) {
        ontologies += Ontology(result->value(0).toString(), result->value(1).toString());
    }

    foreach(Ontology o, ontologies) {
        o.setClasses(readClasses(o));

        QMap<QString, Literal> literals;

        foreach(const Class &c, o.classes()) {
            if ("xsd" == o.prefix()) {
                continue;
            }

            static const QStringList reservedClasses = QStringList()
                    << "http://www.tracker-project.org/ontologies/tracker#Namespace"
                    << "http://www.tracker-project.org/ontologies/tracker#Ontology"
                    << "http://www.w3.org/1999/02/22-rdf-syntax-ns#Property"
                    << "http://www.w3.org/2000/01/rdf-schema#Class"
                    << "http://www.w3.org/2000/01/rdf-schema#Resource";

            if (reservedClasses.contains(c.iri())) {
                continue;
            }

            foreach(const Literal &l, readLiterals(c)) {
                literals.insert(l.name(), l);
            }
        }

        o.setLiterals(literals.values());
    }

    foreach(Ontology o, ontologies) {
        o.setProperties(readProperties(o, overrides));
    }

    return ontologies;
}

static QString
cxxName(const QString &rdfName)
{
    static const QStringList reservedWords =
            QStringList() << "class" << "double" << "Ontology";

    QString cxxName = rdfName;

    cxxName.replace('-', '_');

    if (reservedWords.contains(cxxName)) {
        cxxName += '_';
    }

    return cxxName;
}

static QString
cxxName(const Class &rdfClass)
{
    return cxxName(rdfClass.ontology().prefix()) + "::" + cxxName(rdfClass.name());
}

static void
dumpForwardDeclaration(QTextStream &out, const Class &c)
{
    out << "        class " << cxxName(c.name()) << ";\n";
}

static void
dumpFinalDeclaration(QTextStream &out, const Class &c)
{
    out << "\n"
           "        class " << cxxName(c.name()) << "\n"
           "        {\n"
           "        public:\n"
           "            static const char * encodedIri()\n"
           "            {\n"
           "                return \"" << c.iri() << "\";\n"
           "            }\n"
           "\n"
           "            static const QString & prefixedName()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(\"" << c.ontology().prefix() << ":" << c.name() << "\");\n"
           "                return iri;\n"
           "            }\n"
           "\n"
           "            static const QString & iri()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(encodedIri()); return iri;\n"
           "            }\n"
           "\n"
           "            static const ResourceValue & resource()\n"
           "            {\n"
           "                static const ResourceValue resource(prefixedName(), ResourceValue::PrefixedName);\n"
           "                return resource;\n"
           "            }\n"
           "\n"
           "            typedef rdfs::Class ResourceType;\n"
           "            typedef Ontologies::" << c.ontology().prefix() << " Ontology;\n"
           "        };\n";
}

static void
dumpFinalDeclaration(QTextStream &out, const Property &p)
{
    out << "\n"
           "        class " << cxxName(p.name()) << "\n"
           "        {\n"
           "        public:\n"
           "            static const char * encodedIri()\n"
           "            {\n"
           "                return \"" << p.iri() << "\";\n"
           "            }\n"
           "\n"
           "            static const QString & prefixedName()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(\"" << p.ontology().prefix() << ":" << p.name() << "\");\n"
           "                return iri;\n"
           "            }\n"
           "\n"
           "            static const QString & iri()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(encodedIri()); return iri;\n"
           "            }\n"
           "\n"
           "            static const ResourceValue & resource()\n"
           "            {\n"
           "                static const ResourceValue resource(prefixedName(), ResourceValue::PrefixedName);\n"
           "                return resource;\n"
           "            }\n"
           "\n"
           "            static const PredicateFunction & function()\n"
           "            {\n"
           "                static const PredicateFunction function(resource()); return function;\n"
           "            }\n"
           "\n"
           "            static bool singleValued()\n"
           "            {\n"
           "                return " << (p.singleValued() ? "true" : "false") << ";\n"
           "            }\n"
           "\n"
           "            typedef rdf::Property ResourceType;\n"
           "            typedef " << cxxName(p.domain()) << " Domain;\n"
           "            typedef " << cxxName(p.range()) << " Range;\n"
           "        };\n";
}

static void
dumpFinalDeclaration(QTextStream &out, const Literal &l)
{
    out << "\n"
           "        class " << cxxName(l.name()) << "\n"
           "        {\n"
           "        public:\n"
           "            static const char * encodedIri()\n"
           "            {\n"
           "                return \"" << l.iri() << "\";\n"
           "            }\n"
           "\n"
           "            static const QString & prefixedName()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(\"" << l.ontology().prefix() << ":" << l.name() << "\");\n"
           "                return iri;\n"
           "            }\n"
           "\n"
           "            static const QString & iri()\n"
           "            {\n"
           "                static const QString iri = QString::fromLatin1(encodedIri()); return iri;\n"
           "            }\n"
           "\n"
           "            static const ResourceValue & resource()\n"
           "            {\n"
           "                static const ResourceValue resource(prefixedName(), ResourceValue::PrefixedName); return resource;\n"
           "            }\n"
           "\n"
           "            typedef " << cxxName(l.type()) << " ResourceType;\n"
           "        };\n";
}

static void
dumpOntology(QTextStream &out, const Ontology &ontology, const QString &programName)
{
    const QString prefix = cxxName(ontology.prefix());
    const QString guard = "CUBI_ONTOLOGIES_" % ontology.prefix().toUpper() % "_H";

    QSet<QString> requiredPrefixes;

    foreach(const Property &p, ontology.properties()) {
        requiredPrefixes += p.domain().ontology().prefix();
        requiredPrefixes += p.range().ontology().prefix();
    }

    requiredPrefixes += QLatin1String("rdf");
    requiredPrefixes += QLatin1String("rdfs");
    requiredPrefixes -= ontology.prefix();

    QStringList sortedPrefixList = requiredPrefixes.toList();

    qSort(sortedPrefixList);

    out << "// Generated file (by " << programName << "), DO NOT EDIT.\n"
           "\n"
           "#ifndef " << guard << "\n"
           "#define " << guard << " 1\n"
           "\n"
           "#include <cubi.h>\n"
           "\n"
           "#ifndef CUBI_USE_NAMESPACE_ONTOLOGIES\n"
           "#define CUBI_USE_NAMESPACE_ONTOLOGIES using namespace CUBI_NAMESPACE::Ontologies;\n"
           "#endif\n"
           "\n"
           "#ifndef CUBI_USE_NAMESPACE_RESOURCES\n"
           "#define CUBI_USE_NAMESPACE_RESOURCES using namespace CUBI_NAMESPACE::Resources;\n"
           "#endif\n"
           "\n"
           "CUBI_BEGIN_NAMESPACE\n"
           "\n"
           "namespace Ontologies\n"
           "{\n"
           "    class " << prefix << "\n"
           "    {\n"
           "    public:\n"
           "        static const char * encodedIri()\n"
           "        {\n"
           "            return \"" << ontology.iri() << "\";\n"
           "        }\n"
           "\n"
           "        static const QString & iri()\n"
           "        {\n"
           "            static const QString iri = QString::fromLatin1(encodedIri()); return iri;\n"
           "        }\n"
           "\n"
           "        static QString iri(const QString & suffix)\n"
           "        {\n"
           "            return QLatin1String(encodedIri()) + suffix;\n"
           "        }\n"
           "    };\n"
           "}\n"
           "\n"
           "namespace Resources\n"
           "{\n"
           "    namespace " << prefix << "\n"
           "    {\n";

    foreach(const Class &c, ontology.classes()) {
        dumpForwardDeclaration(out, c);
    }

    out << "    }\n"
           "}\n"
           "\n"
           "CUBI_END_NAMESPACE\n"
           "\n";

    foreach(const QString &p, sortedPrefixList) {
        out << "#include \"" << p << ".h\"\n";
    }

    out << "\n"
           "CUBI_BEGIN_NAMESPACE\n"
           "\n"
           "namespace Resources\n"
           "{\n"
           "    namespace " << prefix << "\n"
           "    {\n";

    foreach(const Class &c, ontology.classes()) {
        dumpFinalDeclaration(out, c);
    }

    foreach(const Property &p, ontology.properties()) {
        dumpFinalDeclaration(out, p);
    }

    foreach(const Literal &l, ontology.literals()) {
        dumpFinalDeclaration(out, l);
    }

    out << "    }\n"
           "}\n"
           "\n"
           "CUBI_END_NAMESPACE\n"
           "\n"
           "#endif // " << guard << "\n";
}

static bool
writeOntologiesHeader(const QDir &outdir, const QList<Ontology> &ontologies, const QString &programName)
{
    QFile file(outdir.filePath("ontologies.h"));

    if (not file.open(QFile::WriteOnly)) {
        err() << file.fileName() << ": " << file.errorString() << endl;
        return false;
    }

    QTextStream out(&file);

    out << "// Generated file (by " << programName << "), DO NOT EDIT.\n"
           "\n"
           "#ifndef CUBI_ONTOLOGIES_H\n"
           "#define CUBI_ONTOLOGIES_H 1\n"
           "\n";

    foreach(const Ontology &o, ontologies) {
        out << "#include \"ontologies/" << o.prefix() << ".h\"\n";
    }

    out << "\n"
           "#endif // CUBI_ONTOLOGIES_H\n";

    return true;
}

static bool
writeOntologiesProject(const QDir &outdir, const QList<Ontology> &ontologies, const QString &programName)
{
    QFile file(outdir.filePath("ontologies.pri"));

    if (not file.open(QFile::WriteOnly)) {
        err() << file.fileName() << ": " << file.errorString() << endl;
        return false;
    }

    QTextStream out(&file);

    out << "# Generated file (by " << programName << "), DO NOT EDIT.\n"
        << "\n"
        << "ontology.headers = \\\n";


    foreach(const Ontology &o, ontologies) {
        out << "    ontologies/" << o.prefix() << ".h \\\n";
    }

    out << "    ontologies.h\n";

    return true;
}

static void
usage()
{
    QTextStream(stdout) << "Usage: " << qAppName() << " [options]" << endl
                        << "  Creates a set of C++ header files from Tracker ontologies" << endl
                        << "Options:" << endl
                        << "  -o, --output-dir  Set the output directory for generated files" << endl
                        << "  -O, --overrides   Set the path to the overrides files" << endl;
}

static bool
parseParameters(const QStringList &arguments, QHash<QString, QString> &parameters)
{
    QStringList::ConstIterator it = arguments.constBegin();

    if (it == arguments.constEnd()) {
        return true;
    }

    it++;

    for (; it != arguments.constEnd(); ++it) {
        if (*it == QLatin1String("-h") || *it == QLatin1String("--help")) {
            usage();
            return false;
        }

        if (*it == "-o" || *it == "--output-dir") {
            it++;

            if (it == arguments.constEnd()) {
                err() << "Missing output dir parameter" << endl;
                return false;
            }

            parameters.insert(settingOutputDir, *it);
            continue;
        }

        if (*it == "-O" || *it == "--overrides") {
            it++;

            if (it == arguments.constEnd()) {
                err() << "Missing overrides file parameter" << endl;
                return false;
            }

            parameters.insert(settingOverrides, *it);
            continue;
        }

        err() << "Invalid parameter " << *it << endl;
        return false;
    }

    return true;
}

int
main(int argc, char **argv)
{
    QCoreApplication app(argc, argv);
    const QStringList args = app.arguments();
    QHash<QString, QString> parameters;

    const QString programName = QFileInfo(args[0]).fileName();

    if (not parseParameters(args, parameters)) {
        return EXIT_FAILURE;
    }

    const QDir srcdir = QDir::current();
    const QString overridesPath = parameters.value(settingOverrides,
                                                   srcdir.filePath(programName + QLatin1String(".overrides")));
    const QDir outdir = parameters.contains(settingOutputDir) ? QDir(parameters.value(settingOutputDir))
                                                              : QDir::current();
    const QDir clsdir(outdir.filePath("ontologies"));

    if (not sparql.isValid()) {
        err() << "Cannot open direct tracker connection. Aborting." << endl;
        return EXIT_FAILURE;
    }

    QSettings overrides(overridesPath, QSettings::IniFormat);
    const QList<Ontology> ontologies = readOntologies(overrides);

    if (not outdir.mkpath(".")) {
        err() << outdir.path() << ": Cannot create output directory. Aborting." << endl;
        return EXIT_FAILURE;
    }

    if (not clsdir.mkpath(".")) {
        err() << outdir.path() << ": Cannot create output directory. Aborting." << endl;
        return EXIT_FAILURE;
    }

    if (not writeOntologiesHeader(outdir, ontologies, programName) ||
        not writeOntologiesProject(outdir, ontologies, programName)) {
        return EXIT_FAILURE;
    }

    foreach(const Ontology &o, ontologies) {
        QFile file(clsdir.filePath(o.prefix() + QLatin1String(".h")));

        if (not file.open(QFile::WriteOnly)) {
            err() << file.fileName() << ": " << file.errorString() << endl;
            continue;
        }

        QTextStream out(&file);
        dumpOntology(out, o, programName);
    }

    return EXIT_SUCCESS;
}
